@@ -21,7 +21,7 @@
 //=============================================================================
 
 #include <stdio.h>
-//#include <stdint.h>
+#include <stdint.h>
 //#include "config.h"
 #include "al.h"
 #include "dsp.h"
