@@ -33,7 +33,9 @@
 #include <QMainWindow>
 #include <QRect>
 #include <QString>
-#include <QPointer>
+#include <QList>
+#include <QMap>
+#include <QMetaObject>
 
 #include <list>
 #include <time.h>
@@ -42,6 +44,14 @@
 #include <unistd.h>
 #endif
 
+// NOTE: When closing top level windows we MUST wait until they have deleted before proceeding.
+//       The problem is that close() calls deleteLater() - NOT delete. There are two ways to handle this.
+//       The simplest method that works is using QCoreApplication::sendPostedEvents().
+//       But advice from Qt bugs said not to do that.
+//       So instead we will use a safer but much more complicated method of waiting for destroyed() signals.
+//       See MusE::clearSong() for details.
+//       This define switches between the two methods, in case there are any problems with either.
+//#define USE_SENDPOSTEDEVENTS_FOR_TOPWIN_CLOSE
 
 // Forward declarations:
 class QCloseEvent;
@@ -53,6 +63,7 @@ class QTimer;
 class QMdiSubWindow;
 class MuseMdiArea;
 class QDockWidget;
+class QAction;
 
 namespace MusECore {
 class AudioOutput;
@@ -99,6 +110,11 @@ class CpuToolbar;
 class CpuStatusBar;
 class SnooperDialog;
 class MasterEdit;
+class MidiEditor;
+class ListEdit;
+class PianoRoll;
+class WaveEdit;
+class DrumEdit;
 
 #define MENU_ADD_SYNTH_ID_BASE 0x8000
 
@@ -110,6 +126,45 @@ class MasterEdit;
 class MusE : public QMainWindow
 {
     Q_OBJECT
+
+#ifndef USE_SENDPOSTEDEVENTS_FOR_TOPWIN_CLOSE
+    class LoadingFinishStruct
+    {
+      public:
+        enum Type { LoadProjectFile, LoadProjectFile1, ClearSong, LoadTemplate, LoadDefaultTemplate, FileClose };
+        enum Flag { NoFlag = 0x0, SongTemplate = 0x1, DoReadMidiPorts = 0x2, ClearAll = 0x4, RestartSequencer = 0x8 };
+        typedef int Flags;
+
+        Type _type;
+        Flags _flags;
+        QString _fileName;
+
+        LoadingFinishStruct(Type type, Flags flags = NoFlag, const QString &fileName = QString());
+    };
+    QList<LoadingFinishStruct> _loadingFinishStructList;
+    struct ObjectDestructionStruct
+    {
+      QMetaObject::Connection _conn;
+      bool _waitForDelete;
+      ObjectDestructionStruct(const QMetaObject::Connection& conn, bool waitForDelete = false);
+    };
+    class ObjectDestructions : public QMap<QObject*, ObjectDestructionStruct>
+    {
+      public:
+        // findWait: True: Search only those marked with wait. False: Search only those NOT marked with wait.
+        // Use normal find() to search all.
+        Iterator findObject(QObject* obj, bool findWait);
+        ConstIterator findObject(QObject* obj, bool findWait) const;
+        // Returns true if there are any objects marked as waiting.
+        bool hasWaitingObjects() const;
+        // Sets or clears all as to wait. Returns true if not empty (has any waiting objects).
+        bool markAll(bool asWait);
+    };
+
+    ObjectDestructions _pendingObjectDestructions;
+    bool _busyWithLoading;
+#endif
+
     enum {CMD_FOLLOW_NO, CMD_FOLLOW_JUMP, CMD_FOLLOW_CONTINUOUS };
 
     //File menu items:
@@ -129,7 +184,7 @@ class MusE : public QMainWindow
     // File menu actions
     QAction *fileSaveAction, *fileOpenAction, *fileNewAction, *fileNewFromTemplateAction;
     QAction *fileSaveRevisionAction, *fileSaveAsAction, *fileSaveAsNewProjectAction, *fileSaveAsTemplateAction;
-    QAction *fileImportMidiAction, *fileExportMidiAction;
+    QAction *fileImportMidiAction, *fileExportMidiAction, *fileExportMidiSelectedVisibleAction, *fileExportSelectedPartsAction;
     QAction *fileImportPartAction, *fileImportWaveAction, *fileMoveWaveFiles, *quitAction;
     QAction *fileCloseAction;
     QAction *editSongInfoAction;
@@ -238,7 +293,6 @@ class MusE : public QMainWindow
     ArrangerView* arrangerView;
     MidiTransformerDialog* midiTransformerDialog;
     QMenu* openRecent;
-    QPointer<MasterEdit> masterEditor;
 
     bool writeTopwinState;
     bool dockMixerA;
@@ -263,7 +317,7 @@ class MusE : public QMainWindow
     MusECore::PartList* getMidiPartsToEdit();
     MusECore::Part* readPart(MusECore::Xml& xml);
     bool checkRegionNotNull();
-    void loadProjectFile1(const QString&, bool songTemplate, bool doReadMidiPorts);
+    bool loadProjectFile1(const QString&, bool songTemplate, bool doReadMidiPorts);
     // Write global configuration.
     void writeGlobalConfiguration(int level, MusECore::Xml&) const;
     // Write song specific configuration.
@@ -276,8 +330,8 @@ class MusE : public QMainWindow
     void addTabbedDock(Qt::DockWidgetArea area, QDockWidget *widget);
     void saveStateExtra();
     void saveStateTopLevels();
-    bool findOpenEditor(const TopWin::ToplevelType type, MusECore::PartList* pl);
-    bool findOpenListEditor(MusECore::PartList* pl);
+    MusEGui::MidiEditor* findOpenEditor(const TopWin::ToplevelType type, MusECore::PartList* pl = nullptr);
+    MusEGui::ListEdit* findOpenListEditor(MusECore::PartList* pl);
     bool filterInvalidParts(const TopWin::ToplevelType type, MusECore::PartList* pl);
     void updateStatusBar();
     void setAndAdjustFonts();
@@ -293,6 +347,16 @@ class MusE : public QMainWindow
     int avrCpuLoadCounter;
     float fCurCpuLoad;
 
+#ifndef USE_SENDPOSTEDEVENTS_FOR_TOPWIN_CLOSE
+    bool finishLoadProjectFile(bool restartSequencer);
+    bool finishLoadProjectFile1(const QString& name, bool songTemplate, bool doReadMidiPorts);
+    bool finishClearSong(bool clear_all);
+    void finishLoadTemplate();
+    void finishLoadDefaultTemplate();
+    void finishFileClose(bool restartSequencer);
+    void executeLoadingFinish();
+#endif
+
 signals:
     void configChanged();
     void activeTopWinChanged(MusEGui::TopWin*);
@@ -315,7 +379,7 @@ private slots:
     void importMidi();
     void importWave();
     void importPart();
-    void exportMidi();
+    void exportMidi(bool selectedVisibleTracksOnly = false, bool selectedPartsOnly = false, bool alignPartsToStart = false);
     void findUnusedWaveFiles();
 
     void toggleTransport(bool);
@@ -377,7 +441,11 @@ public slots:
     void bounceToTrack(MusECore::AudioOutput* ao = nullptr);
     void closeEvent(QCloseEvent*event) override;
     void loadProjectFile(const QString&);
-    void loadProjectFile(const QString&, bool songTemplate, bool doReadMidiPorts);
+#ifdef USE_SENDPOSTEDEVENTS_FOR_TOPWIN_CLOSE
+    bool loadProjectFile(const QString&, bool songTemplate, bool doReadMidiPorts);
+#else
+    bool loadProjectFile(const QString&, bool songTemplate, bool doReadMidiPorts, bool *doRestartSequencer = nullptr);
+#endif
     void fileClose();
     void toplevelDeleting(MusEGui::TopWin* tl);
     bool seqRestart();
@@ -397,19 +465,21 @@ public slots:
     void startEditor(MusECore::PartList*, int);
     void startScoreQuickly();
     void startPianoroll(bool newwin = false);
-    void startPianoroll(MusECore::PartList* pl, bool showDefaultCtrls = false, bool newwin = false);
+    MusEGui::PianoRoll* startPianoroll(
+      MusECore::PartList* pl, bool showDefaultCtrls = false, bool newwin = false, bool *createdNotFound = nullptr);
     void startWaveEditor(bool newwin = false);
-    void startWaveEditor(MusECore::PartList*, bool newwin = false);
+    MusEGui::WaveEdit* startWaveEditor(MusECore::PartList*, bool newwin = false, bool *createdNotFound = nullptr);
     void openInScoreEdit(ScoreEdit* destination, MusECore::PartList* pl, bool allInOne=false);
     void openInScoreEdit(ScoreEdit* destination, bool allInOne=false);
     void openInScoreEdit_allInOne(QWidget* destination);
     void openInScoreEdit_oneStaffPerTrack(QWidget* destination);
-    void startMasterEditor();
+    MusEGui::MasterEdit* startMasterEditor(bool *createdNotFound = nullptr);
     void showMasterList(bool);
     void startListEditor(bool newwin = false);
     void startListEditor(MusECore::PartList*, bool newwin = false);
     void startDrumEditor(bool newwin = false);
-    void startDrumEditor(MusECore::PartList* pl, bool showDefaultCtrls = false, bool newwin = false);
+    MusEGui::DrumEdit* startDrumEditor(
+      MusECore::PartList* pl, bool showDefaultCtrls = false, bool newwin = false, bool *createdNotFound = nullptr);
     void startEditor(MusECore::Track*);
     void startMidiTransformer();
     void startSnooper();
@@ -432,8 +502,13 @@ public slots:
     void addProjectToRecentList(const QString& name);
     void saveProjectRecentList();
 
+#ifndef USE_SENDPOSTEDEVENTS_FOR_TOPWIN_CLOSE
+    void objectDestroyed(QObject *obj);
+#endif
+
 public:
     MusE();
+    virtual ~MusE();
 
     void populateAddTrack();
 
@@ -499,6 +574,11 @@ public:
 #ifdef HAVE_LASH
     void lash_idle_cb ();
 #endif
+
+#ifndef USE_SENDPOSTEDEVENTS_FOR_TOPWIN_CLOSE
+    void addPendingObjectDestruction(QObject*);
+#endif
+
 };
 
 //extern void addProject(const QString& name);
