@@ -534,7 +534,7 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       connect(canvas,  SIGNAL(horizontalScrollNoLimit(unsigned)),hscroll, SLOT(setPosNoLimit(unsigned))); 
       connect(time,    SIGNAL(timeChanged(unsigned)),   SLOT(setTime(unsigned)));
 
-      connect(list, SIGNAL(verticalScrollSetYpos(int)), vscroll, SLOT(setValue(int)));
+      connect(list, SIGNAL(verticalScrollSetYpos(unsigned)), this, SLOT(verticalScrollSetYpos(unsigned)));
 
       connect(canvas, SIGNAL(tracklistChanged()), list, SLOT(tracklistChanged()));
       connect(canvas, SIGNAL(dclickPart(MusECore::Track*)), SIGNAL(editPart(MusECore::Track*)));
@@ -547,7 +547,7 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       connect(canvas, SIGNAL(dropMidiFile(const QString&)), SIGNAL(dropMidiFile(const QString&)));
 
       connect(canvas, SIGNAL(toolChanged(int)), SIGNAL(toolChanged(int)));
-      connect(MusEGlobal::song,   SIGNAL(controllerChanged(MusECore::Track*, int)), SLOT(controllerChanged(MusECore::Track*, int)));
+      connect(MusEGlobal::song, &MusECore::Song::controllerChanged, this, &Arranger::controllerChanged);
 
       connect(_rasterCombo, &RasterLabelCombo::rasterChanged, [this](int raster) { rasterChanged(raster); } );
 
@@ -712,6 +712,8 @@ void Arranger::configChanged()
       gridOnButton->setChecked(MusEGlobal::config.canvasShowGrid);
       gridOnButton->blockSignals(false);
 
+      canvas->setAutomationPointRadius(MusEGlobal::config.audioAutomationPointRadius);
+
       canvas->redraw();
       }
 
@@ -756,8 +758,8 @@ void Arranger::songChanged(MusECore::SongChangedStruct_t type)
               {
                 if(!MusEGlobal::song->trackExists(t))
                 {
-                  delete w;
                   trackInfoWidget->addWidget(nullptr, 1);
+                  delete w;
                   selected = nullptr;
                   switchInfo(0);
                 } 
@@ -774,8 +776,8 @@ void Arranger::songChanged(MusECore::SongChangedStruct_t type)
               {
                 if(!MusEGlobal::song->trackExists(t))
                 {
-                  delete w;
                   trackInfoWidget->addWidget(nullptr, 2);
+                  delete w;
                   selected = nullptr;
                   switchInfo(0);
                 } 
@@ -825,8 +827,15 @@ void Arranger::songChanged(MusECore::SongChangedStruct_t type)
                    SC_TRACK_MOVED | SC_TRACK_RESIZED |
                    SC_PART_INSERTED | SC_PART_REMOVED | SC_PART_MODIFIED | 
                    SC_SIG | SC_TEMPO | SC_MASTER)) 
+        {
           canvas->updateItems();
-        
+          canvas->updateAudioAutomation();
+        }
+        else if(type & (SC_AUDIO_CONTROLLER | SC_AUDIO_CONTROLLER_LIST | SC_AUDIO_CONTROLLER_SELECTION))
+        {
+          canvas->updateAudioAutomation();
+        }
+
         if(type & (SC_PART_SELECTION))
         {
           // Prevent race condition: Ignore if the change was ultimately sent by the canvas itself.
@@ -860,7 +869,8 @@ void Arranger::songChanged(MusECore::SongChangedStruct_t type)
         // Try these:
         if(type & (SC_PART_INSERTED | SC_PART_REMOVED | SC_PART_MODIFIED |
                    SC_EVENT_INSERTED | SC_EVENT_REMOVED | SC_EVENT_MODIFIED |
-                   SC_CLIP_MODIFIED | SC_MARKER_INSERTED | SC_MARKER_REMOVED | SC_MARKER_MODIFIED))
+                   SC_CLIP_MODIFIED | SC_MARKER_INSERTED | SC_MARKER_REMOVED | SC_MARKER_MODIFIED |
+                   SC_AUDIO_CTRL_MOVE_MODE))
         canvas->redraw();
         
         // We must marshall song changed instead of connecting to the strip's song changed
@@ -877,7 +887,8 @@ void Arranger::songChanged(MusECore::SongChangedStruct_t type)
         //  selectionChanged(), itemsAreSelected() has the latest citems' selected flags.
         if(type & (SC_TRACK_SELECTION | SC_PART_SELECTION | 
                   SC_TRACK_INSERTED | SC_TRACK_REMOVED | SC_TRACK_MODIFIED | 
-                  SC_PART_INSERTED | SC_PART_REMOVED | SC_PART_MODIFIED))
+                  SC_PART_INSERTED | SC_PART_REMOVED | SC_PART_MODIFIED |
+                  SC_AUDIO_CONTROLLER | SC_AUDIO_CONTROLLER_LIST | SC_AUDIO_CONTROLLER_SELECTION))
           _parentWin->selectionChanged();
     }
 
@@ -1097,6 +1108,9 @@ void Arranger::cmd(int cmd)
             case CMD_INSERT_EMPTYMEAS:
                   ncmd = PartCanvas::CMD_INSERT_EMPTYMEAS;
                   break;
+            case CMD_DELETE:
+                  ncmd = PartCanvas::CMD_DELETE;
+                  break;
             default:
                   return;
             }
@@ -1187,21 +1201,22 @@ void Arranger::verticalScrollSetYpos(unsigned ypos)
 
 void Arranger::clear()
       {
+
       {
         AudioStrip* w = static_cast<AudioStrip*>(trackInfoWidget->getWidget(1));
+        trackInfoWidget->addWidget(nullptr, 1);
         if (w)
-              delete w;
-        trackInfoWidget->addWidget(0, 1);
+          delete w;
       }
-      
+
       {
         MidiStrip* w = static_cast<MidiStrip*>(trackInfoWidget->getWidget(2));
+        trackInfoWidget->addWidget(nullptr, 2);
         if (w)
-              delete w;
-        trackInfoWidget->addWidget(0, 2);
+          delete w;
       }
-      
-      selected = 0;
+
+      selected = nullptr;
       }
 
 //void Arranger::wheelEvent(QWheelEvent* ev)
@@ -1209,9 +1224,10 @@ void Arranger::clear()
 //      emit redirectWheelEvent(ev);
 //      }
 
-void Arranger::controllerChanged(MusECore::Track *t, int ctrlId)
+void Arranger::controllerChanged(
+  const MusECore::Track *t, int ctrlId, unsigned int frame, MusECore::CtrlGUIMessage::Type type)
 {
-      canvas->controllerChanged(t, ctrlId);
+      canvas->controllerChanged(t, ctrlId, frame, type);
 }
 
 //---------------------------------------------------------
@@ -1277,10 +1293,10 @@ void Arranger::switchInfo(int n)
             MidiStrip* w = static_cast<MidiStrip*>(trackInfoWidget->getWidget(2));
             if (w)
             {
+              trackInfoWidget->addWidget(nullptr, 2);
               //fprintf(stderr, "Arranger::switchInfo audio strip: deleting midi strip\n");
               delete w;
               //w->deleteLater();
-              trackInfoWidget->addWidget(nullptr, 2);
             }
           }
           {
@@ -1288,6 +1304,7 @@ void Arranger::switchInfo(int n)
               if (w == nullptr || selected != w->getTrack()) {
                     if (w)
                     {
+                          trackInfoWidget->addWidget(nullptr, 1);
                           //fprintf(stderr, "Arranger::switchInfo deleting strip\n");
                           delete w;
                           //w->deleteLater();
@@ -1305,7 +1322,7 @@ void Arranger::switchInfo(int n)
                     //  and the strip's songChanged() seems to be called before Arranger songChanged()
                     //  gets called and has a chance to stop the crash.
                     //connect(MusEGlobal::song, SIGNAL(songChanged(MusECore::SongChangedStruct_t)), w, SLOT(songChanged(MusECore::SongChangedStruct_t)));
-                    
+
                     connect(MusEGlobal::muse, SIGNAL(configChanged()), w, SLOT(configChanged()));
                     w->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
                     trackInfoWidget->addWidget(w, 1);
@@ -1320,10 +1337,10 @@ void Arranger::switchInfo(int n)
             AudioStrip* w = static_cast<AudioStrip*>(trackInfoWidget->getWidget(1));
             if (w)
             {
+              trackInfoWidget->addWidget(nullptr, 1);
               //fprintf(stderr, "Arranger::switchInfo midi strip: deleting audio strip\n");
               delete w;
               //w->deleteLater();
-              trackInfoWidget->addWidget(nullptr, 1);
             }
           }
           {
@@ -1331,6 +1348,7 @@ void Arranger::switchInfo(int n)
             if (w == nullptr || selected != w->getTrack()) {
                   if (w)
                   {
+                        trackInfoWidget->addWidget(nullptr, 2);
                         //fprintf(stderr, "Arranger::switchInfo deleting strip\n");
                         delete w;
                         //w->deleteLater();
@@ -1344,7 +1362,7 @@ void Arranger::switchInfo(int n)
 
                   // No. See above.
                   //connect(MusEGlobal::song, SIGNAL(songChanged(MusECore::SongChangedStruct_t)), w, SLOT(songChanged(MusECore::SongChangedStruct_t)));
-                  
+
                   connect(MusEGlobal::muse, SIGNAL(configChanged()), w, SLOT(configChanged()));
                   w->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
                   trackInfoWidget->addWidget(w, 2);
@@ -1353,7 +1371,7 @@ void Arranger::switchInfo(int n)
                   }
           }
         }
-            
+
       if (trackInfoWidget->curIdx() == n)
             return;
       trackInfoWidget->raiseWidget(n);
@@ -1392,6 +1410,8 @@ void Arranger::keyPressEvent(QKeyEvent* event)
         key += Qt::ALT;
   if (((QInputEvent*)event)->modifiers() & Qt::ControlModifier)
         key+= Qt::CTRL;
+  if (((QInputEvent*)event)->modifiers() & Qt::MetaModifier)
+        key+= Qt::META;
 
   RasterizerModel::RasterPick rast_pick = RasterizerModel::NoPick;
   const int cur_rast = rasterVal();
@@ -1399,15 +1419,16 @@ void Arranger::keyPressEvent(QKeyEvent* event)
   if (key == shortcuts[SHRT_ZOOM_IN].key) {
         horizontalZoom(true, QCursor::pos());
         return;
-        }
+  }
   else if (key == shortcuts[SHRT_ZOOM_OUT].key) {
         horizontalZoom(false, QCursor::pos());
         return;
-        }
+  }
   else if (key == shortcuts[SHRT_HIDE_MIXER_STRIP].key) {
       showTrackInfo(!showTrackinfoFlag);
       return;
   }
+
   // QUANTIZE shortcuts from midi editors is reused for SNAP in Arranger
   //    `does not work exactly the same but close enough I think.
   else if (key == shortcuts[SHRT_SET_QUANT_BAR].key) {
@@ -1510,6 +1531,10 @@ int Arranger::selectionSize() const { return canvas->selectionSize(); }
 bool Arranger::itemsAreSelected() const { return canvas->itemsAreSelected(); }
 void Arranger::songIsClearing() const { canvas->songIsClearing(); }
 
+bool Arranger::isSingleAudioAutomationSelection() const { return canvas->isSingleAudioAutomationSelection(); }
+int Arranger::audioAutomationSelectionSize() const  { return canvas->audioAutomationSelectionSize(); }
+bool Arranger::audioAutomationItemsAreSelected() const  { return canvas->audioAutomationItemsAreSelected(); }
+
 //---------------------------------------------------------
 //   setupHZoomRange
 //---------------------------------------------------------
@@ -1520,4 +1545,68 @@ void Arranger::setupHZoomRange()
   hscroll->setScaleRange(min, _maxXMag);
 }
 
+void Arranger::toggleTrackHeights()
+{
+    const int defaultTrackHeight = MusEGlobal::config.trackHeight;
+    const int alternateTrackHeight = MusEGlobal::config.trackHeightAlternate;
+
+    // check if tracks have different height
+    int height = MusEGlobal::song->tracks()->front()->height();
+    bool differentHeight = false;
+    MusECore::Track* selTrack = nullptr;
+    for (auto track : *MusEGlobal::song->tracks())
+    {
+        if (height != track->height())
+        {
+            differentHeight = true;
+        }
+
+        if (track->selected())
+        {
+            selTrack = track; // if multiple tracks are selected we simply keep the last one
+        }
+    }
+    if (differentHeight)
+    {
+        for (auto track : *MusEGlobal::song->tracks())
+        {
+            track->setHeight(defaultTrackHeight);
+        }
+    }
+    else
+    {
+        if (height == defaultTrackHeight)
+        {
+            for (auto track : *MusEGlobal::song->tracks())
+            {
+                track->setHeight(alternateTrackHeight);
+            }
+        }
+        else
+        {
+            for (auto track : *MusEGlobal::song->tracks())
+            {
+                track->setHeight(defaultTrackHeight);
+            }
+        }
+    }
+
+    list->adjustScrollbar();
+    updateTracklist();
+    list->update();
+    MusEGlobal::song->update(SC_TRACK_RESIZED);
+
+    if(selTrack)
+    {
+        // force update Ypos before scrolling
+        int newYpos = selTrack->y() - list->height() - 21;
+        if (newYpos < 0)
+            newYpos = 0;
+        list->setYPos(newYpos);
+        list->scrollToTrack(selTrack);
+    }
+
+}
+
+
 } // namespace MusEGui
