@@ -31,11 +31,9 @@
 #include "type_defs.h"
 #include "tools.h"
 
+#include "ctrl.h"
 
 // Forward declarations:
-//class QKeyEvent;
-//class QPoint;
-//class QComboBox;
 class QScrollBar;
 class QVBoxLayout;
 class QHBoxLayout;
@@ -140,7 +138,6 @@ class Arranger : public QWidget {
       void setTempo100();
       void setTempo200();
       void gridOnChanged(bool v);
-      void verticalScrollSetYpos(unsigned);
       void horizontalZoom(bool zoom_in, const QPoint& glob_pos);
       void horizontalZoom(int mag, const QPoint& glob_pos);
       void updateTracklist();
@@ -162,11 +159,17 @@ class Arranger : public QWidget {
       void setTool(int);
       void updateTrackInfo(MusECore::SongChangedStruct_t flags);
       void configChanged();
-      void controllerChanged(MusECore::Track *t, int ctrlId);
+      void controllerChanged(
+        const MusECore::Track *t, int CtrlId,
+        unsigned int frame = 0, MusECore::CtrlGUIMessage::Type type = MusECore::CtrlGUIMessage::PAINT_UPDATE);
       void focusCanvas();
+      void verticalScrollSetYpos(unsigned);
+      void toggleTrackHeights();
+
 
    public:
-      enum { CMD_CUT_PART, CMD_COPY_PART, CMD_COPY_PART_IN_RANGE, CMD_PASTE_PART, CMD_PASTE_CLONE_PART,
+      enum { CMD_DELETE,
+             CMD_CUT_PART, CMD_COPY_PART, CMD_COPY_PART_IN_RANGE, CMD_PASTE_PART, CMD_PASTE_CLONE_PART,
              CMD_PASTE_PART_TO_TRACK, CMD_PASTE_CLONE_PART_TO_TRACK, CMD_PASTE_DIALOG, CMD_INSERT_EMPTYMEAS };
       
       const int arrangerTools = PointerTool | PencilTool | RubberTool | CutTool | GlueTool | MuteTool |
@@ -207,6 +210,9 @@ class Arranger : public QWidget {
       bool isSingleSelection() const;
       int selectionSize() const;
       bool itemsAreSelected() const;
+      bool isSingleAudioAutomationSelection() const;
+      int audioAutomationSelectionSize() const;
+      bool audioAutomationItemsAreSelected() const;
       void setGlobalTempo(int);
       void clear();
       void songIsClearing() const;
