@@ -62,6 +62,7 @@
 #include "tlist.h"
 #include "synth.h"
 #include "pcanvas.h"
+#include "part.h"
 
 // Forwards from header:
 #include <QCloseEvent>
@@ -74,6 +75,7 @@
 #include "xml.h"
 #include "tools.h"
 #include "partcolortoolbar.h"
+#include "automation_mode_toolbar.h"
 
 namespace MusEGui {
 
@@ -110,6 +112,12 @@ ArrangerView::ArrangerView(QWidget* parent)
   // Make sure name doesn't conflict with other TopWin edit toolbar object names.
   editTools->setObjectName("arrangerTools");
 
+  automationModeToolBar = new AutomationModeToolBar(this);
+  automationModeToolBar->setInterpolateMode(MusEGlobal::config.audioAutomationDrawDiscrete ? 0 : 1);
+  automationModeToolBar->setBoxMode(MusEGlobal::config.audioAutomationShowBoxes ? 0 : 1);
+  automationModeToolBar->setOptimize(MusEGlobal::config.audioAutomationOptimize);
+  addToolBar(automationModeToolBar);
+
   addToolBar(visTracks);
 
   partColorToolBar = new PartColorToolbar(this);
@@ -129,8 +137,10 @@ ArrangerView::ArrangerView(QWidget* parent)
   connect(arranger, SIGNAL(toolChanged(int)), editTools, SLOT(set(int)));
   connect(MusEGlobal::muse, SIGNAL(configChanged()), arranger, SLOT(configChanged()));
   connect(arranger, SIGNAL(setUsedTool(int)), editTools, SLOT(set(int)));
-  connect(MusEGlobal::muse, &MusE::configChanged, editTools, &EditToolBar::configChanged);
-
+  _configChangedEditToolsMetaConn = connect(MusEGlobal::muse, &MusE::configChanged, editTools, &EditToolBar::configChanged);
+  connect(automationModeToolBar, &AutomationModeToolBar::interpolateModeChanged, [this](int mode) { automationInterpolateModeChanged(mode); } );
+  connect(automationModeToolBar, &AutomationModeToolBar::boxModeChanged, [this](int mode) { automationBoxModeChanged(mode); } );
+  connect(automationModeToolBar, &AutomationModeToolBar::optimizeChanged, [this](bool v) { automationOptimizeChanged(v); } );
 
 
   //-------- Edit Actions
@@ -327,10 +337,10 @@ ArrangerView::ArrangerView(QWidget* parent)
   //---------------------------------------------------
   //  Connect script receiver
   //---------------------------------------------------
-  connect(&scripts,
+  _deliveredScriptReceivedMetaConn = connect(&scripts,
           &MusECore::Scripts::execDeliveredScriptReceived,
           [this](int id) { execDeliveredScript(id); } );
-  connect(&scripts,
+  _userScriptReceivedMetaConn = connect(&scripts,
           &MusECore::Scripts::execUserScriptReceived,
           [this](int id) { execUserScript(id); } );
 
@@ -341,6 +351,12 @@ ArrangerView::ArrangerView(QWidget* parent)
                           MusEGui::shortcuts[MusEGui::SHRT_HIDE_MIXER_STRIP].key);
   menuSettings->addAction(tr("Configure &Custom Columns..."), this, SLOT(configCustomColumns()));
 
+  QAction *trackHeightAction = new QAction(tr("&Toggle Track Heights"), this);
+  trackHeightAction->setShortcut(shortcuts[SHRT_TOGGLE_TRACK_HEIGHTS].key);
+  connect(trackHeightAction, SIGNAL(triggered()), arranger, SLOT(toggleTrackHeights()));
+
+  menuSettings->addAction(trackHeightAction);
+
   openAddTrackMenuAction = new QAction(this);
   addAction(openAddTrackMenuAction);
   connect(openAddTrackMenuAction, &QAction::triggered, [=](){ arranger->getTrackList()->openAddTrackMenu(); });
@@ -413,7 +429,9 @@ ArrangerView::ArrangerView(QWidget* parent)
 
 ArrangerView::~ArrangerView()
 {
-  
+  disconnect(_configChangedEditToolsMetaConn);
+  disconnect(_deliveredScriptReceivedMetaConn);
+  disconnect(_userScriptReceivedMetaConn);
 }
 
 void ArrangerView::closeEvent(QCloseEvent* e)
@@ -643,7 +661,7 @@ void ArrangerView::cmd(int cmd)
         arranger->cmd(Arranger::CMD_INSERT_EMPTYMEAS);
         break;
     case CMD_DELETE:
-        MusECore::delete_selected_parts();
+        arranger->cmd(Arranger::CMD_DELETE);
         break;
     case CMD_DELETE_TRACK:
         MusEGlobal::audio->msgRemoveTracks();
@@ -1049,12 +1067,14 @@ void ArrangerView::clipboardChanged()
 
 //---------------------------------------------------------
 //   selectionChanged
-//   NOTE: This is received upon EITHER a part or track selection change from the Arranger.
+//   NOTE: This is received upon a part or track or
+//          audio controller selection change from the Arranger.
 //---------------------------------------------------------
 
 void ArrangerView::selectionChanged()
       {
       bool pflag = arranger->itemsAreSelected();
+      bool acflag = arranger->audioAutomationItemsAreSelected();
       bool tflag = MusECore::tracks_are_selected();
 
       editDeleteSelectedTrackAction->setEnabled(tflag);
@@ -1063,16 +1083,15 @@ void ArrangerView::selectionChanged()
       editMoveDownSelTrackAction->setEnabled(tflag);
       editMoveTopSelTrackAction->setEnabled(tflag);
       editMoveBottomSelTrackAction->setEnabled(tflag);
-   
-      editDeleteAction->setEnabled(pflag);
-      editCutAction->setEnabled(pflag);
-      editCopyAction->setEnabled(pflag);
+
+      editDeleteAction->setEnabled(pflag || acflag);
+      editCutAction->setEnabled(pflag || acflag);
+      editCopyAction->setEnabled(pflag || acflag);
       editShrinkPartsAction->setEnabled(pflag);
       editExpandPartsAction->setEnabled(pflag);
       editCleanPartsAction->setEnabled(pflag);
       }
 
-
 void ArrangerView::updateVisibleTracksButtons()
 {
   visTracks->updateVisibleTracksButtons();
@@ -1144,5 +1163,37 @@ void ArrangerView::execUserScript(int id)
                                     MusECore::getSelectedMidiParts(), 0, false); // TODO: get quant from arranger
 }
 
+//---------------------------------------------------------
+//   automationInterpolateModeChanged
+//---------------------------------------------------------
+void ArrangerView::automationInterpolateModeChanged(int m)
+{
+  if(m == 0)
+    MusEGlobal::config.audioAutomationDrawDiscrete = true;
+  else if(m == 1)
+    MusEGlobal::config.audioAutomationDrawDiscrete = false;
+}
+
+//---------------------------------------------------------
+//   automationBoxModeChanged
+//---------------------------------------------------------
+void ArrangerView::automationBoxModeChanged(int m)
+{
+  if(m == 0)
+    MusEGlobal::config.audioAutomationShowBoxes = true;
+  else if(m == 1)
+    MusEGlobal::config.audioAutomationShowBoxes = false;
+  // Update to redraw any points.
+  if(getArranger() && getArranger()->getCanvas())
+    getArranger()->getCanvas()->update();
+}
+
+//---------------------------------------------------------
+//   automationOptimizeChanged
+//---------------------------------------------------------
+void ArrangerView::automationOptimizeChanged(bool v)
+{
+  MusEGlobal::config.audioAutomationOptimize = v;
+}
 
 } // namespace MusEGui
