@@ -29,6 +29,7 @@
 #include "cobject.h"
 #include "scripts.h"
 
+#include <QMetaObject>
 
 // Forward declarations:
 class QCloseEvent;
@@ -48,6 +49,7 @@ class VisibleTracks;
 class PartColorToolbar;
 class Arranger;
 class Xml;
+class AutomationModeToolBar;
 
 class ArrangerView : public TopWin
 {
@@ -75,6 +77,7 @@ private:
     EditToolBar* editTools;
     VisibleTracks* visTracks;
     PartColorToolbar *partColorToolBar;
+    AutomationModeToolBar* automationModeToolBar;
 
     Arranger* arranger;
 
@@ -115,6 +118,10 @@ private:
     MusECore::Scripts scripts;
     QMenu* menuScripts;
 
+    QMetaObject::Connection _configChangedEditToolsMetaConn;
+    QMetaObject::Connection _deliveredScriptReceivedMetaConn;
+    QMetaObject::Connection _userScriptReceivedMetaConn;
+
 private slots:
     void globalCut();
     void globalInsert();
@@ -130,6 +137,9 @@ private slots:
     void toggleMixerStrip();
     void execDeliveredScript(int);
     void execUserScript(int);
+    void automationInterpolateModeChanged(int);
+    void automationBoxModeChanged(int);
+    void automationOptimizeChanged(bool);
 
 signals:
     void isDeleting(MusEGui::TopWin*);
