@@ -102,7 +102,7 @@ extern uint64_t curTimeUS();
 const char* seqMsgList[] = {
       "SEQM_REVERT_OPERATION_GROUP", "SEQM_EXECUTE_OPERATION_GROUP",
       "SEQM_EXECUTE_PENDING_OPERATIONS", 
-      "SEQM_RESET_DEVICES", "SEQM_INIT_DEVICES", "SEQM_PANIC",
+      "SEQM_RESET_DEVICES", "SEQM_PANIC",
       "SEQM_MIDI_LOCAL_OFF",
       "SEQM_PLAY_MIDI_EVENT",
       "SEQM_SET_HW_CTRL_STATE",
@@ -111,18 +111,11 @@ const char* seqMsgList[] = {
       "SEQM_SET_AUX",
       "SEQM_UPDATE_SOLO_STATES",
       "AUDIO_ROUTEADD", "AUDIO_ROUTEREMOVE", "AUDIO_REMOVEROUTES",
-      "AUDIO_ADDPLUGIN",
       "AUDIO_SET_PREFADER", "AUDIO_SET_CHANNELS",
       "AUDIO_SWAP_PLUGINS",
-      "AUDIO_CLEAR_CONTROLLER_EVENTS",
       "AUDIO_SEEK_PREV_AC_EVENT",
       "AUDIO_SEEK_NEXT_AC_EVENT",
-      "AUDIO_ERASE_AC_EVENT",
-      "AUDIO_ERASE_RANGE_AC_EVENTS",
-      "AUDIO_ADD_AC_EVENT",
-      "AUDIO_CHANGE_AC_EVENT",
       "AUDIO_SET_SEND_METRONOME", 
-      "AUDIO_START_MIDI_LEARN",
       "MS_PROCESS", "MS_STOP", "MS_SET_RTC", "MS_UPDATE_POLL_FD",
       "SEQM_IDLE", "SEQM_SEEK",
       "AUDIO_WAIT"
@@ -588,7 +581,7 @@ void Audio::reSyncAudio()
     //  either via changes to tempo map or the static tempo value. The actual transport frame is allowed
     //  to continue progressing naturally but our representation of it (_pos) jumps to a new value
     //  so that the relation between ticks and frames in all our tempo routines remains correct.
-    // But the relation betweeen actual transport frame and current tick will still be incorrect
+    // But the relation between actual transport frame and current tick will still be incorrect
     //  for the given new tempo value, that is why this adjustment to our _pos is needed.
     // Another solution would be to actually seek the transport to the correct frame.
     // In either solution any waves will jump to a new position but midi will continue on.
@@ -1450,45 +1443,20 @@ void Audio::processMsg(AudioMsg* msg)
             case AUDIO_SET_CHANNELS:
                   msg->snode->setChannels(msg->ival);
                   break;
-            case AUDIO_ADDPLUGIN:
-                  msg->snode->addPlugin(msg->plugin, msg->ival);
-                  break;
             case AUDIO_SWAP_PLUGINS:
                   msg->snode->swapPlugins(msg->a, msg->b);
                   break;
-            case AUDIO_CLEAR_CONTROLLER_EVENTS:
-                  msg->snode->clearControllerEvents(msg->ival);
-                  break;
             case AUDIO_SEEK_PREV_AC_EVENT:
                   msg->snode->seekPrevACEvent(msg->ival);
                   break;
             case AUDIO_SEEK_NEXT_AC_EVENT:
                   msg->snode->seekNextACEvent(msg->ival);
                   break;
-            case AUDIO_ERASE_AC_EVENT:
-                  msg->snode->eraseACEvent(msg->ival, msg->a);
-                  break;
-            case AUDIO_ERASE_RANGE_AC_EVENTS:
-                  msg->snode->eraseRangeACEvents(msg->ival, msg->a, msg->b);
-                  break;
-            case AUDIO_ADD_AC_EVENT:
-                  msg->snode->addACEvent(msg->ival, msg->a, msg->dval);
-                  break;
-            case AUDIO_CHANGE_AC_EVENT:
-                  msg->snode->changeACEvent(msg->ival, msg->a, msg->b, msg->dval);
-                  break;
 
             case AUDIO_SET_SEND_METRONOME:
                   msg->snode->setSendMetronome((bool)msg->ival);
                   break;
             
-            case AUDIO_START_MIDI_LEARN:
-                  // Reset the values. The engine will fill these from driver events.
-                  MusEGlobal::midiLearnPort = -1;
-                  MusEGlobal::midiLearnChan = -1;
-                  MusEGlobal::midiLearnCtrl = -1;
-                  break;
-
             case SEQM_RESET_DEVICES:
                   for (int i = 0; i < MusECore::MIDI_PORTS; ++i)                         
                   {      
@@ -1496,9 +1464,6 @@ void Audio::processMsg(AudioMsg* msg)
                       MusEGlobal::midiPorts[i].instrument()->reset(i);
                   }      
                   break;
-            case SEQM_INIT_DEVICES:
-                  initDevices(msg->a);
-                  break;
             case SEQM_MIDI_LOCAL_OFF:
                   sendLocalOff();
                   break;
@@ -1565,10 +1530,12 @@ void Audio::seek(const Pos& p)
       // The transport (and user) must NOT be allowed to influence the position during bounce mode.
       if(!bounce())
       {
+        bool alreadyThere = false;
         // This is expected and required sometimes, for example to
         //  sync the prefetch system to a position we are ALREADY at.
         // But being fairly costly, not too frequently. So we warn just in case.
         if (_pos == p) {
+              alreadyThere = true;
               if(MusEGlobal::debugMsg)
                 fprintf(stderr, "Audio::seek already at frame:%u\n", p.frame());
               }
@@ -1609,10 +1576,18 @@ void Audio::seek(const Pos& p)
           //  part moving etc. Without a force, the wrong data was being played.  Tim 08/17/08
           // This does not wait.
           // FIXME: Actually it WILL until the the message is sent, but which is usually right away.
+          // Diagnostics.
+          //fprintf(stderr, "Audio::seek: pos:%d\n", _pos.frame());
           MusEGlobal::audioPrefetch->msgSeek(_pos.frame(), true);
         }
               
         write(sigFd, "G", 1);   // signal seek to gui
+        // Signal enable all controllers and clear all track automation record lists to gui.
+        // NOT if we're already there.
+        // This is to preserve controller enabled states and any initial value stored while in
+        //  automation 'write' mode while transport stopped.
+        if(!alreadyThere)
+          write(sigFd, "N", 1);
       }
       }
 
@@ -1878,7 +1853,8 @@ void Audio::recordStop(bool restart, Undo* ops)
             if (track->recordFlag() || MusEGlobal::song->bounceTrack == track) {
                   MusEGlobal::song->cmdAddRecordedWave(track, startRecordPos, restart ? _pos : endRecordPos, operations);
                   if(!restart)
-                    operations.push_back(UndoOp(UndoOp::SetTrackRecord, track, false, true)); // True = non-undoable.
+                    operations.push_back(UndoOp(
+                      UndoOp::SetTrackRecord, track, false, double(0), double(0), double(0), double(0), true)); // True = non-undoable.
                   }
             }
       MidiTrackList* ml = MusEGlobal::song->midis();
@@ -1910,7 +1886,8 @@ void Audio::recordStop(bool restart, Undo* ops)
         {            
           MusEGlobal::song->bounceOutput = nullptr;
           ao->setRecFile(nullptr); // if necessary, this automatically deletes _recFile
-          operations.push_back(UndoOp(UndoOp::SetTrackRecord, ao, false, true));  // True = non-undoable.
+          operations.push_back(UndoOp(
+            UndoOp::SetTrackRecord, ao, false, double(0), double(0), double(0), double(0), true));  // True = non-undoable.
         }
       }  
       MusEGlobal::song->bounceTrack = nullptr;
