@@ -51,6 +51,8 @@
 // For debugging output: Uncomment the fprintf section.
 #define ERROR_COBJECT(dev, format, args...)  fprintf(dev, format, ##args)
 #define DEBUG_COBJECT(dev, format, args...) // fprintf(dev, format, ##args)
+// For debugging song clearing and loading: Uncomment the fprintf section.
+#define DEBUG_LOADING_AND_CLEARING(dev, format, args...) // fprintf(dev, format, ##args);
 
 using std::list;
 using MusEGlobal::muse;
@@ -208,12 +210,24 @@ TopWin::TopWin(ToplevelType t, QWidget* parent, const char* name, Qt::WindowFlag
 //    {
 //        setParent(nullptr);
 //    }
+
+    DEBUG_LOADING_AND_CLEARING(stderr, "TopWin::TopWin:%p <%s>\n", this, typeName(_type).toUtf8().constData());
+
+#ifndef USE_SENDPOSTEDEVENTS_FOR_TOPWIN_CLOSE
+    // Inform when destroyed.
+    // We do not want the ArrangerView included here.
+    if(_type != ARRANGER)
+      MusEGlobal::muse->addPendingObjectDestruction(this);
+#endif
+
 }
 
 TopWin::~TopWin()
 {
     DEBUG_COBJECT(stderr, "TopWin dtor: %s\n", objectName().toLatin1().constData());
 
+    DEBUG_LOADING_AND_CLEARING(stderr, "~TopWin:%p <%s>\n", this, typeName(_type).toUtf8().constData());
+
     // Toolbars must be deleted explicitly to avoid memory leakage and corruption.
     // For some reason (toolbar sharing?) they are reparented and thus
     // not destroyed by the original parent topwin when closed.
@@ -270,18 +284,32 @@ void TopWin::readStatus(MusECore::Xml& xml)
             {
                 if (!sharesToolsAndMenu())
                 {
-                    if (!restoreState(QByteArray::fromHex(xml.parse1().toLatin1())))
+                    // We can only restore the state with version-compatible data.
+                    // If toolbars were altered, 'alien' loaded data will not fit!
+                    if(xml.isVersionEqualToLatest())
                     {
-                        fprintf(stderr,"ERROR: couldn't restore toolbars. trying default configuration...\n");
-                        if (!restoreState(_toolbarNonsharedInit[_type]))
-                            fprintf(stderr,"ERROR: couldn't restore default toolbars. this is not really a problem.\n");
+                      if (!restoreState(QByteArray::fromHex(xml.parse1().toLatin1())))
+                      {
+                          fprintf(stderr,"ERROR: couldn't restore toolbars. trying default configuration...\n");
+                          if (!restoreState(_toolbarNonsharedInit[_type]))
+                              fprintf(stderr,"ERROR: couldn't restore default toolbars. this is not really a problem.\n");
+                      }
                     }
+                    else
+                      xml.parse1();
                 }
                 else
                 {
-                    _savedToolbarState = QByteArray::fromHex(xml.parse1().toLatin1());
-                    if (_savedToolbarState.isEmpty())
-                        _savedToolbarState=_toolbarNonsharedInit[_type];
+                    // We can only restore the state with version-compatible data.
+                    // If toolbars were altered, 'alien' loaded data will not fit!
+                    if(xml.isVersionEqualToLatest())
+                    {
+                      _savedToolbarState = QByteArray::fromHex(xml.parse1().toLatin1());
+                      if (_savedToolbarState.isEmpty())
+                          _savedToolbarState=_toolbarNonsharedInit[_type];
+                    }
+                    else
+                      xml.parse1();
                 }
             }
 //            else if (tag == "shares_menu")
@@ -658,9 +686,25 @@ void TopWin::readConfiguration(ToplevelType t, MusECore::Xml& xml)
             else if (tag == "height")
                 _heightInit[t] = xml.parseInt();
             else if (tag == "nonshared_toolbars")
-                _toolbarNonsharedInit[t] = QByteArray::fromHex(xml.parse1().toLatin1());
+            {
+                // We can only restore the state with version-compatible data.
+                // If toolbars were altered, 'alien' loaded data will not fit!
+                if(xml.isVersionEqualToLatest())
+
+                  _toolbarNonsharedInit[t] = QByteArray::fromHex(xml.parse1().toLatin1());
+                else
+                  xml.parse1();
+            }
             else if (tag == "shared_toolbars")
-                _toolbarSharedInit[t] = QByteArray::fromHex(xml.parse1().toLatin1());
+            {
+                // We can only restore the state with version-compatible data.
+                // If toolbars were altered, 'alien' loaded data will not fit!
+                if(xml.isVersionEqualToLatest())
+                  _toolbarSharedInit[t] = QByteArray::fromHex(xml.parse1().toLatin1());
+
+                else
+                  xml.parse1();
+            }
             else if (tag == "default_subwin")
                 _openTabbed[t] = xml.parseInt();
             else
@@ -728,7 +772,11 @@ QString TopWin::typeName(ToplevelType t)
     switch (t)
     {
     case PIANO_ROLL: return tr("Piano roll");
-//    case LISTE: return tr("List editor");
+#ifdef MOVE_LISTEDIT_FROM_DOCK_TO_WINDOW_PULL1099
+    case LISTE: return tr("List editor");
+#else
+    //case LISTE: return tr("List editor");
+#endif
     case DRUM: return tr("Drum editor");
     case MASTER: return tr("Master track editor");
     case WAVE: return tr("Wave editor");
@@ -777,6 +825,14 @@ void TopWin::setWindowTitle (const QString& title)
 
 void TopWin::storeSettings() {}
 
+void TopWin::setOpenInNewWin(bool newwin)
+{
+    if ( !(_openTabbed[_type]) || (_openTabbed[_type] && newwin) )
+        setIsMdiWin(false);
+    else
+        setIsMdiWin(true);
+}
+
 //void TopWin::windowStateChanged(Qt::WindowStates oldState, Qt::WindowStates newState)
 //{
 //    // Due to bug in Oxygen and Breeze at least on *buntu 16.04 LTS and some other distros,
