@@ -185,6 +185,8 @@ void GlobalSettingsConfig::updateSettings()
       preferKnobsVsSlidersCheckBox->setChecked(MusEGlobal::config.preferKnobsVsSliders);
       showControlValuesCheckBox->setChecked(MusEGlobal::config.showControlValues);
       monitorOnRecordCheckBox->setChecked(MusEGlobal::config.monitorOnRecord);
+      momentaryMuteCheckBox->setChecked(MusEGlobal::config.momentaryMute);
+      momentarySoloCheckBox->setChecked(MusEGlobal::config.momentarySolo);
       lineEditStyleHackCheckBox->setChecked(MusEGlobal::config.lineEditStyleHack);
       showNoteNamesCheckBox->setChecked(MusEGlobal::config.showNoteNamesInPianoRoll);
       showNoteTooltipsCheckBox->setChecked(MusEGlobal::config.showNoteTooltips);
@@ -198,6 +200,7 @@ void GlobalSettingsConfig::updateSettings()
       sendNullParamsCB->setChecked(MusEGlobal::config.midiSendNullParameters);      
       optimizeControllersCB->setChecked(MusEGlobal::config.midiOptimizeControllers);      
       guiRefreshSelect->setValue(MusEGlobal::config.guiRefresh);
+      audioAutomationPointRadius->setValue(MusEGlobal::config.audioAutomationPointRadius);
       minSliderSelect->setValue(int(MusEGlobal::config.minSlider));
       minMeterSelect->setValue(MusEGlobal::config.minMeter);
       freewheelCheckBox->setChecked(MusEGlobal::config.freewheelMode);
@@ -246,6 +249,7 @@ void GlobalSettingsConfig::updateSettings()
       unhideTracksCheckBox->setChecked(MusEGlobal::config.unhideTracks);
 
       trackHeight->setValue(MusEGlobal::config.trackHeight);
+      trackHeightAlternate->setValue(MusEGlobal::config.trackHeightAlternate);
 
       lv2UiBehaviorComboBox->setCurrentIndex(static_cast<int>(MusEGlobal::config.lv2UiBehavior));
 
@@ -294,6 +298,7 @@ void GlobalSettingsConfig::apply()
       {
       int rtcticks       = rtcResolutionSelect->currentIndex();
       MusEGlobal::config.guiRefresh  = guiRefreshSelect->value();
+      MusEGlobal::config.audioAutomationPointRadius  = audioAutomationPointRadius->value();
       MusEGlobal::config.minSlider   = minSliderSelect->value();
       MusEGlobal::config.minMeter    = minMeterSelect->value();
       MusEGlobal::config.freewheelMode = freewheelCheckBox->isChecked();
@@ -342,6 +347,8 @@ void GlobalSettingsConfig::apply()
       MusEGlobal::config.preferKnobsVsSliders = preferKnobsVsSlidersCheckBox->isChecked();
       MusEGlobal::config.showControlValues = showControlValuesCheckBox->isChecked();
       MusEGlobal::config.monitorOnRecord = monitorOnRecordCheckBox->isChecked();
+      MusEGlobal::config.momentaryMute = momentaryMuteCheckBox->isChecked();
+      MusEGlobal::config.momentarySolo = momentarySoloCheckBox->isChecked();
       MusEGlobal::config.lineEditStyleHack = lineEditStyleHackCheckBox->isChecked();
       MusEGlobal::config.showNoteNamesInPianoRoll = showNoteNamesCheckBox->isChecked();
       MusEGlobal::config.showNoteTooltips = showNoteTooltipsCheckBox->isChecked();
@@ -372,6 +379,7 @@ void GlobalSettingsConfig::apply()
         MusEGlobal::midiSeq->msgSetRtc();        // set midi tick rate
       
       MusEGlobal::config.trackHeight = trackHeight->value();
+      MusEGlobal::config.trackHeightAlternate = trackHeightAlternate->value();
 
       MusEGlobal::config.lv2UiBehavior = static_cast<MusEGlobal::CONF_LV2_UI_BEHAVIOR>(lv2UiBehaviorComboBox->currentIndex());
 
