@@ -419,7 +419,7 @@ void View::paint(const QRect& r, const QRegion& rg)
 //         fprintf(stderr, "  #%d: x:%d y:%d w:%d h:%d\n", rg_r_cnt, rg_r.x(), rg_r.y(), rg_r.width(), rg_r.height());
 //       }   
 
-      p.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform | QPainter::HighQualityAntialiasing, false);
+      p.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform, false);
       
       if (bgPixmap.isNull())
             p.fillRect(rr, brush);
@@ -1027,15 +1027,8 @@ QPoint View::mapDev(const QPoint& r) const
 
 void View::mapDev(const QRegion& rg_in, QRegion& rg_out) const
 {
-#if QT_VERSION >= 0x050800
-  for(QRegion::const_iterator i = rg_in.begin(); i != rg_in.end(); ++i)
-    rg_out += mapDev(*i);
-#else  
-  const QVector<QRect> rects = rg_in.rects();
-  const int sz = rects.size();
-  for(int i = 0; i < sz; ++i)
-    rg_out += mapDev(rects.at(i));
-#endif
+  for (const auto& it : rg_in)
+    rg_out += mapDev(it);
 }
 
 #if 0
@@ -1172,15 +1165,8 @@ QPoint View::map(const QPoint& p) const
 
 void View::map(const QRegion& rg_in, QRegion& rg_out) const
 {
-#if QT_VERSION >= 0x050800
-  for(QRegion::const_iterator i = rg_in.begin(); i != rg_in.end(); ++i)
-    rg_out += map(*i);
-#else  
-  const QVector<QRect> rects = rg_in.rects();
-  const int sz = rects.size();
-  for(int i = 0; i < sz; ++i)
-    rg_out += map(rects.at(i));
-#endif
+  for (const auto& it : rg_in)
+    rg_out += map(it);
 }
       
 int View::mapx(int x) const
