@@ -53,10 +53,10 @@ namespace MusEGlobal {
  */
 
 StripConfig::StripConfig()
-  { _serial = -1; _tmpFileIdx = -1; _visible = true; _width = -1; _deleted = false; }
-StripConfig::StripConfig(int trackSerial, bool visible, int width)
-  { _serial = trackSerial; _tmpFileIdx = -1; _visible = visible; _width = width; _deleted = false; }
-bool StripConfig::isNull() const { return _serial < 0; }
+  { _tmpFileIdx = -1; _visible = true; _width = -1; _deleted = false; }
+StripConfig::StripConfig(const QUuid& trackUuid, bool visible, int width)
+  { _uuid = trackUuid; _tmpFileIdx = -1; _visible = visible; _width = width; _deleted = false; }
+bool StripConfig::isNull() const { return _uuid.isNull(); }
 
 GlobalConfigValues config = {
       QStringList(),              // pluginLadspaPathList
@@ -278,7 +278,7 @@ GlobalConfigValues config = {
       QString(""),                  // copyright string for smf export
       1,                            // smf export file format
       false,                        // midi export file 2 byte timesigs instead of 4
-      true,                         // optimize midi export file note offs
+      false,                        // optimize midi export file note offs
       true,                         // expRunningStatus; Save space by using running status
       true,                         // Split imported tracks into multiple parts.
       false,                         // useLastEditedEvent
@@ -292,6 +292,7 @@ GlobalConfigValues config = {
       QString("GM"),                // importMidiDefaultInstr Default to this instrument not Generic, if no match found
       true,                         // exportDrumMapOverrides Apply drum map overrides to export
       true,                         // exportChannelOverridesToNewTrack Drum map Channel overrides go to a separate track
+      false,                        // exportSelectedPartsAlignToBar0 When exporting only selected parts, align their leftmost position with bar 0
       1,                            // startMode
       QString(""),                  // start song path
       false,                        // startSongLoadConfig
@@ -363,6 +364,7 @@ GlobalConfigValues config = {
       true,                         // unhideTracks
       true,                         // smartFocus
       24,                           // trackHeight
+      100,                          // trackHeightAlternate
       true,                         // borderlessMouse
       false,                        // autoSave
       false,                        // scrollableSubMenus
@@ -373,11 +375,14 @@ GlobalConfigValues config = {
       false,                        // preferKnobsVsSliders Whether to prefer the use of knobs over sliders, esp in mixer.
       true,                         // showControlValues Whether to show the value along with label in small controls, esp in mixer.
       true,                         // monitorOnRecord  Whether to automatically monitor on record arm.
+      false,                        // momentaryMute Whether the track mute buttons are momentary or toggle.
+      false,                        // momentarySolo Whether the track solo buttons are momentary or toggle.
       true,                         // lineEditStyleHack Force line edit widgets to draw a frame at small sizes. Some styles refuse to draw the frame.
       false,                        // preferMidiVolumeDb Prefer midi volume as decibels instead of 0-127.
       true,                         // midiCtrlGraphMergeErase Whether to erase underlying erase target items when dragging/dropping source items.
       false,                        // midiCtrlGraphMergeEraseInclusive Whether to erase target items in-between source item groups.
       true,                         // midiCtrlGraphMergeEraseWysiwyg Whether to erase past the last item in a group to include its original source width.
+      MusECore::CtrlList::PasteErase, // audioCtrlGraphPasteEraseOptions;
       MusEGlobal::RoutePreferSecondAlias,  // preferredRouteNameOrAlias
       true,                         // routerExpandVertically
       2,                            // routerGroupingChannels
@@ -396,7 +401,11 @@ GlobalConfigValues config = {
       true,                         // showStatusBar
       6,                             // recentListLength - number of files in recent menu
       false,
-      false
+      false,
+      true,                         // audioAutomationDrawDiscrete
+      true,                         // audioAutomationShowBoxes
+      true,                         // audioAutomationOptimize
+      2                             // audioAutomationPointRadius
 };
 
 } // namespace MusEGlobal
