@@ -33,6 +33,9 @@
 #include <QString>
 #include <QList>
 #include <QStringList>
+#include <QUuid>
+
+#include "ctrl.h"
 
 namespace MusECore {
 class Xml;
@@ -78,8 +81,8 @@ enum RouteNameAliasPreference { RoutePreferCanonicalName = 0, RoutePreferFirstAl
 enum WaveDrawing { WaveRmsPeak=1, WaveOutLine=2 };
 
 struct StripConfig {
-  // The corresponding track's serial number. Can be -1.
-  int _serial;
+  // The corresponding track's uuid. Can be null.
+  QUuid _uuid;
   // The corresponding track's index in the song file. Can be -1.
   // Temporary during loading to avoid using globally or locally
   //  'unique' identifiers in the song file, such as the serial,
@@ -91,7 +94,7 @@ struct StripConfig {
   bool _deleted;
 
   StripConfig();
-  StripConfig(int trackSerial, bool visible, int width);
+  StripConfig(const QUuid& trackUuid, bool visible, int width);
 
   bool isNull() const;
 
@@ -328,6 +331,7 @@ struct GlobalConfigValues {
       QString importMidiDefaultInstr;  // Default to this instrument not Generic, if no match found
       bool exportDrumMapOverrides; // Apply Port, Channel, and ANote drum map overrides to export
       bool exportChannelOverridesToNewTrack; // Drum map Channel overrides go to a separate track
+      bool exportSelectedPartsAlignToBar0;   // When exporting only selected parts, align their leftmost position with bar 0.
 
       int startMode;          // 0 - start with last song
                               // 1 - start with default template
@@ -389,6 +393,7 @@ struct GlobalConfigValues {
 //       drumTrackPreference_t drumTrackPreference;
       bool smartFocus;
       int trackHeight;
+      int trackHeightAlternate;
       bool borderlessMouse;
       bool autoSave;
       bool scrollableSubMenus;
@@ -399,12 +404,15 @@ struct GlobalConfigValues {
       bool preferKnobsVsSliders; // Whether to prefer the use of knobs over sliders, esp in mixer.
       bool showControlValues; // Whether to show the value along with label in small controls, esp in mixer.
       bool monitorOnRecord;  // Whether to automatically monitor on record arm.
+      bool momentaryMute; // Whether the track mute buttons toggle or are momentary or toggle.
+      bool momentarySolo; // Whether the track solo buttons toggle or are momentary or toggle.
       bool lineEditStyleHack; // Force line edit widgets to draw a frame at small sizes. Some styles refuse to draw the frame.
       bool preferMidiVolumeDb; // Prefer midi volume as decibels instead of 0-127.
       // NOTE: The following are similar to the paste dialog function options, stored separately.
       bool midiCtrlGraphMergeErase; // Whether to erase underlying erase target items when dragging/dropping source items.
       bool midiCtrlGraphMergeEraseInclusive; // Whether to erase target items in-between source item groups.
       bool midiCtrlGraphMergeEraseWysiwyg; // Whether to erase past the last item in a group to include its original source width.
+      MusECore::CtrlList::PasteEraseOptions audioCtrlGraphPasteEraseOptions;
       RouteNameAliasPreference preferredRouteNameOrAlias;
       bool routerExpandVertically; // Whether to expand the router items vertically. (Good use of space but slow!)
       // How to group the router channels together for easier multi-channel manipulation.
@@ -434,6 +442,11 @@ struct GlobalConfigValues {
 
       bool mixerDockedA;
       bool mixerDockedB;
+
+      bool audioAutomationDrawDiscrete;
+      bool audioAutomationShowBoxes;
+      bool audioAutomationOptimize;
+      int audioAutomationPointRadius;
       };
 
 
