@@ -22,17 +22,12 @@
 //=========================================================
 
 #include <stdio.h>
-//#include <errno.h>
-//#include <unistd.h>
-//#include <stdlib.h>
 
 #include <QActionGroup>
 #include <QDir>
 #include <QFileInfo>
-//#include <QToolButton>
 
 #include "globals.h"
-//#include "config.h"
 
 namespace MusEGlobal {
 
@@ -78,6 +73,10 @@ const int numRtAudioDevices = 6;
 
 MusEGui::Rasterizer *globalRasterizer = nullptr;
 
+// This holds all the unit symbols found. Instead of storing many copies of the same texts
+//  (dB, Hz, frames etc.) all over the place, we store an index into this list.
+MusECore::ValueUnits_t valueUnits;
+
 QTimer* heartBeatTimer;
 
 bool blinkTimerPhase = false;
@@ -281,7 +280,7 @@ int midiInputPorts   = 0;    // receive from all devices
 int midiInputChannel = 0;    // receive all channel
 int midiRecordType   = 0;    // receive all events
 int midiThruType = 0;        // transmit all events
-int midiFilterCtrl1 = 0;
+int midiFilterCtrl1 = 0;     // val 0 means no controller filtered. val >= 1 means controller number (val - 1)
 int midiFilterCtrl2 = 0;
 int midiFilterCtrl3 = 0;
 int midiFilterCtrl4 = 0;
@@ -311,35 +310,21 @@ MusECore::MetronomeSettings metroSongSettings;
 // Whether to use the global or song metronome settings.
 bool metroUseSongSettings = false;
 
-bool rcEnable = false;
-unsigned char rcStopNote = 28;
-unsigned char rcRecordNote = 31;
-unsigned char rcGotoLeftMarkNote = 33;
-unsigned char rcPlayNote = 29;
-unsigned char rcSteprecNote = 36;
-unsigned char rcForwardNote = 26;
-unsigned char rcBackwardNote = 24;
 
-bool rcEnableCC = false;
-unsigned char rcGotoLeftMarkCC = 111;
-unsigned char rcBackwardCC = 112;
-unsigned char rcForwardCC = 113;
-unsigned char rcStopCC = 114;
-unsigned char rcPlayCC = 115;
-//unsigned char rcInsertPauseCC = 116;
-unsigned char rcRecordCC = 117;
+MusECore::MidiRemote midiRemote;
+// Whether the midi remote control dialog box is currently learning.
+bool midiRemoteIsLearning = false;
+// Whether to use the global or song metronome settings.
+bool midiRemoteUseSongSettings = false;
+
+// Whether the midi to audio assignment dialog box is currently learning.
+bool midiToAudioAssignIsLearning = false;
 
 // REMOVE Tim. automation. Remove this.
 // Deprecated. MusEGlobal::automation is now fixed TRUE
 //   for now until we decide what to do with it.
 bool automation = true;
 
-// Midi learn params. These will be initialized to -1 by any midi learn function, 
-//  and then filled by the midi engine in response to the drivers.
-int midiLearnPort = -1;
-int midiLearnChan = -1;
-int midiLearnCtrl = -1;
-
 const QString inputRoutingToolTipBase = QObject::tr("Input routing");
 const QString noInputRoutingToolTipWarn = inputRoutingToolTipBase + QString("\n") + QObject::tr("Warning: No input routes! Click to connect...");
 
@@ -357,9 +342,9 @@ QString defaultStyle = "Fusion";
 //   convertFrame4ProjectSampleRate
 //---------------------------------------------------------
 
-unsigned convertFrame4ProjectSampleRate(unsigned frame)
+unsigned convertFrame4ProjectSampleRate(unsigned frame, unsigned frame_sample_rate)
 {
-  return double(frame) * double(MusEGlobal::sampleRate) / double(MusEGlobal::projectSampleRate);
+  return double(frame) * double(frame_sample_rate) / double(MusEGlobal::projectSampleRate);
 }
 
 //---------------------------------------------------------
