@@ -49,6 +49,7 @@
 #include "audio.h"
 #include "gconfig.h"
 #include "undo.h"
+#include "xml_statistics.h"
 
 using std::set;
 using std::pair;
@@ -68,6 +69,10 @@ void MusE::importMidi()
 
 void MusE::importMidi(const QString &file)
       {
+      // Are we already busy waiting for something while loading or closing another project?
+      if(_busyWithLoading)
+        return;
+
       QString fn;
       if (file.isEmpty()) {
                fn = MusEGui::getOpenFileName(MusEGlobal::lastMidiPath, MusEGlobal::midi_file_pattern, this,
@@ -194,7 +199,7 @@ bool MusE::importMidi(const QString name, bool merge)
             {
               // TEST: Hopefully shouldn't require any routing saves/restorations as in midi port config set device name...
               MusEGlobal::audio->msgSetMidiDevice(mp, md);
-              // TEST: Hopefully can get away with this ouside the loop below...
+              // TEST: Hopefully can get away with this outside the loop below...
               // Save settings. Use simple version - do NOT set style or stylesheet, this has nothing to do with that.
               //MusEGlobal::muse->changeConfig(true);     // save configuration file
               //MusEGlobal::audio->msgUpdateSoloStates();
@@ -618,19 +623,7 @@ void MusE::importPart()
             bool loadAll;
             QString filename = MusEGui::getOpenFileName(QString(""), MusEGlobal::part_file_pattern, this, tr("MusE: load part"), &loadAll);
             if (!filename.isEmpty()){
-                  // Make a backup of the current clone list, to retain any 'copy' items,
-                  //  so that pasting works properly after.
-                  MusECore::CloneList copyCloneList = MusEGlobal::cloneList;
-                  // Clear the clone list to prevent any dangerous associations with
-                  //  current non-original parts.
-                  MusEGlobal::cloneList.clear();
-            
                   importPartToTrack(filename, curPos, track);
-                  
-                  // Restore backup of the clone list, to retain any 'copy' items,
-                  //  so that pasting works properly after.
-                  MusEGlobal::cloneList.clear();
-                  MusEGlobal::cloneList = copyCloneList;
                }   
             }
       else {
@@ -653,7 +646,8 @@ void MusE::importPartToTrack(QString& filename, unsigned tick, MusECore::Track*
         int posOffset = 0;
         int  notDone = 0;
         int  done = 0;
-        
+        MusECore::XmlReadStatistics stats;
+
         bool end = false;
         MusEGlobal::song->startUndo();
         for (;;) 
@@ -670,7 +664,7 @@ void MusE::importPartToTrack(QString& filename, unsigned tick, MusECore::Track*
                   if (tag == "part") {
                         // Read the part.
                         MusECore::Part* p = 0;
-                        p = MusECore::Part::readFromXml(xml, track);
+                        p = MusECore::Part::readFromXml(xml, track, &stats);
                         // If it could not be created...
                         if(!p)
                         {
@@ -692,6 +686,9 @@ void MusE::importPartToTrack(QString& filename, unsigned tick, MusECore::Track*
                         MusEGlobal::song->applyOperation(MusECore::UndoOp(MusECore::UndoOp::AddPart, p),
                                                          MusECore::Song::OperationUndoable);
                         }
+                  else if (tag == "audioTrackAutomation") {
+
+                        }
                   else
                         xml.unknown("MusE::importPartToTrack");
                   break;
