@@ -1478,6 +1478,7 @@ void MidiInstrument::getControllers(MidiControllerList* dest, int channel, int p
 #endif
   mcl = controller();
   // Copy the instrument controller pointers directly to the destination.
+  // The midnam has priority, since this is a map.
   // Defer the RPN update until after done.
   for(imc = mcl->begin(); imc != mcl->end(); ++imc)
     dest->add(imc->second, false);
@@ -1486,6 +1487,22 @@ void MidiInstrument::getControllers(MidiControllerList* dest, int channel, int p
   dest->update_RPN_Ctrls_Reserved();
 }
 
+bool MidiInstrument::RPN_Ctrls_Reserved(int channel, int patch) const
+{
+  bool midnam_rsv = false;
+#ifdef MIDNAM_SUPPORT
+  // Is there a midnam controller list for the given channel and patch?
+  const MidiControllerList* mcl = _midnamDocument.getControllers(channel, patch);
+  // Check if the midnam controller list reserves any of the EIGHT standard General Midi RPN controllers.
+  if(mcl && mcl->RPN_Ctrls_Reserved())
+    midnam_rsv = true;
+#endif
+  // Check if this instrument's controller list reserves any of the EIGHT standard General Midi RPN controllers.
+  // If either the midnam or the instrument contains reserved RPN controllers, then the whole combined
+  //  instrument is considered to reserve them.
+  return midnam_rsv || controller()->RPN_Ctrls_Reserved();
+}
+
 #ifdef _USE_INSTRUMENT_OVERRIDES_
 int MidiInstrument::isWorkingMapItem(int patch, int index, int fields) const
 {
