@@ -248,7 +248,7 @@ void ListEdit::songChanged(MusECore::SongChangedStruct_t type)
             liste->clear();
             for (MusECore::iPart p = _pl->begin(); p != _pl->end(); ++p) {
                 MusECore::MidiPart* part = (MusECore::MidiPart*) (p->second);
-                if (part->sn() == curPartId)
+                if (part->uuid() == curPartId)
                     curPart  = part;
 
                 for (MusECore::ciEvent i = part->events().begin(); i != part->events().end(); ++i) {
@@ -291,7 +291,7 @@ void ListEdit::genPartlist()
             MusECore::PartList* pl = it->parts();
             MusECore::iPart ip;
             for (ip = pl->begin(); ip != pl->end(); ++ip) {
-                if (ip->second->sn() == i) {
+                if (ip->second->uuid() == i) {
                     _pl->add(ip->second);
                     break;
                 }
@@ -486,18 +486,31 @@ QString EventListItem::text(int col) const
 //   ListEdit
 //---------------------------------------------------------
 
+#ifdef MOVE_LISTEDIT_FROM_DOCK_TO_WINDOW_PULL1099
+ListEdit::ListEdit(MusECore::PartList* pl, QWidget* parent, const char* name)
+    : MidiEditor(TopWin::LISTE, 0, pl, parent, name)
+#else
 ListEdit::ListEdit(MusECore::PartList* pl, QWidget* parent)
     : QWidget(parent)
+#endif
 {
     setObjectName("ListEdit");
     _isDeleting = false;
 
     _pl = pl;
     for (const auto& i : *_pl)
-        _pidSet.insert(i.second->sn());
+        _pidSet.insert(i.second->uuid());
 
     selectedTick=0;
 
+#ifdef MOVE_LISTEDIT_FROM_DOCK_TO_WINDOW_PULL1099
+    menuConfig = menuBar()->addMenu(tr("&Display"));
+    menuConfig->menuAction()->setStatusTip(tr("Display menu: Display options specific to current editor."));
+    menuConfig->addAction(subwinAction);
+//    menuConfig->addAction(shareAction);
+    menuConfig->addAction(fullscreenAction);
+#endif
+
     noteAction = new QAction(tr("Note"));
     sysexAction = new QAction(tr("SysEx"));
     ctrlAction = new QAction(tr("Ctrl"));
@@ -634,7 +647,11 @@ ListEdit::ListEdit(MusECore::PartList* pl, QWidget* parent)
     //    Rest
     //---------------------------------------------------
 
+#ifdef MOVE_LISTEDIT_FROM_DOCK_TO_WINDOW_PULL1099
+    //QGridLayout* mainGrid = new QGridLayout(this);
+#else
     QGridLayout* mainGrid = new QGridLayout(this);
+#endif
     mainGrid->setRowStretch(1, 100);
     mainGrid->setColumnStretch(0, 100);
     mainGrid->addWidget(tb, 0, 0);
@@ -644,17 +661,17 @@ ListEdit::ListEdit(MusECore::PartList* pl, QWidget* parent)
     if(pl->empty())
     {
         curPart = nullptr;
-        curPartId = -1;
+        curPartId = QUuid();
     }
     else
     {
         curPart   = (MusECore::MidiPart*)pl->begin()->second;
         if(curPart)
-            curPartId = curPart->sn();
+            curPartId = curPart->uuid();
         else
         {
             curPart = nullptr;
-            curPartId = -1;
+            curPartId = QUuid();
         }
     }
 
