@@ -81,6 +81,8 @@ struct MidiOutputParams {
 //   MidiDevice
 //---------------------------------------------------------
 
+typedef LockFreeMPSCRingBuffer<MidiRecordEvent> MidiRecFifo;
+
 class MidiDevice {
    public:
       // Types of MusE midi devices.
@@ -159,7 +161,7 @@ class MidiDevice {
       LockFreeMPSCRingBuffer<MidiPlayEvent> *_userEventBuffers;
       
       // Recording fifos. To speed up processing, one per channel plus one special system 'channel' for channel-less events like sysex.
-      MidiRecFifo _recordFifo[MusECore::MUSE_MIDI_CHANNELS + 1];   
+      MidiRecFifo *_recordFifo[MusECore::MUSE_MIDI_CHANNELS + 1];
 
       // To hold current output program, and RPN/NRPN parameter numbers and values.
       MidiOutputParams _curOutParamNums[MusECore::MUSE_MIDI_CHANNELS];
@@ -255,6 +257,8 @@ class MidiDevice {
       inline void setrwFlags(int val)         { _rwFlags = val; }
       inline const QString& state() const     { return _state; }
       inline void setState(const QString& s)  { _state = s; }
+      inline bool readEnable() const          { return _readEnable; }
+      inline bool writeEnable() const         { return _writeEnable; }
 
       inline virtual bool isSynti() const     { return false; }
       inline virtual int selectRfd()          { return -1; }
@@ -292,7 +296,7 @@ class MidiDevice {
       void beforeProcess();
       void afterProcess();
       int tmpRecordCount(const unsigned int ch)     { return _tmpRecordCount[ch]; }
-      MidiRecFifo& recordEvents(const unsigned int ch) { return _recordFifo[ch]; }
+      MidiRecFifo *recordEvents(const unsigned int ch) { return _recordFifo[ch]; }
       bool sysexFIFOProcessed()                     { return _sysexFIFOProcessed; }
       void setSysexFIFOProcessed(bool v)            { _sysexFIFOProcessed = v; }
       
@@ -314,14 +318,14 @@ class MidiDevice {
       // Whether this track (and the branch it is in) can force other parallel branches to
       //  increase their latency compensation to match this one.
       // If false, this branch will NOT disturb other parallel branches' compensation,
-      //  intead only allowing compensation UP TO the worst case in other branches.
+      //  instead only allowing compensation UP TO the worst case in other branches.
       virtual bool canDominateOutputLatencyMidi(bool capture) const;
       virtual bool canDominateInputLatencyMidi(bool capture) const;
       // Whether this track (and the branch it is in) can force other parallel branches to
       //  increase their latency compensation to match this one - IF this track is an end-point
       //  and the branch allows domination.
       // If false, this branch will NOT disturb other parallel branches' compensation,
-      //  intead only allowing compensation UP TO the worst case in other branches.
+      //  instead only allowing compensation UP TO the worst case in other branches.
       virtual bool canDominateEndPointLatencyMidi(bool capture) const;
       // Whether this track and its branch can correct for latency, not just compensate.
       inline virtual bool canCorrectOutputLatencyMidi() const { return false; }
@@ -352,7 +356,7 @@ class MidiDevice {
       //  in a branch of the graph.
       virtual TrackLatencyInfo& setCorrectionLatencyInfoMidi(bool capture, bool input, float finalWorstLatency, float callerBranchLatency = 0.0f);
       virtual TrackLatencyInfo& getLatencyInfoMidi(bool capture, bool input);
-      // Used during latency compensation processing. When analyzing in 'reverse' this mechansim is
+      // Used during latency compensation processing. When analyzing in 'reverse' this mechanism is
       //  needed only to equalize the timing of all the AudioOutput tracks.
       // It is applied as a direct offset in the latency delay compensator in getData().
       virtual unsigned long latencyCompWriteOffsetMidi(bool capture) const;
