@@ -35,6 +35,7 @@
 #include <QList>
 #include <QPair>
 #include <QToolTip>
+#include <QUuid>
 
 #include <stdio.h>
 #include <limits.h>
@@ -1097,7 +1098,14 @@ void DrumCanvas::keyPressed(int index, int velocity)
       if (_steprec) /* && pos[0] >= start_tick && pos[0] < end_tick [removed by flo93: this is handled in steprec->record] */
       {
         if ( curPart && instrument_map[index].tracks.contains(curPart->track()) )
-            steprec->record(curPart,instrument_map[index].pitch,ourDrumMap[index].len,editor->raster(),velocity,MusEGlobal::globalKeyState&Qt::ControlModifier,MusEGlobal::globalKeyState&Qt::ShiftModifier, -1 /* invalid pitch as "really played" -> the "insert rest" feature is never triggered */);
+            steprec->record(
+              curPart,
+              instrument_map[index].pitch,
+              ourDrumMap[index].len,
+              editor->raster(),
+              velocity,
+              MusEGlobal::globalKeyState&Qt::ControlModifier,
+              MusEGlobal::globalKeyState&Qt::ShiftModifier);
         else
         {
           QSet<MusECore::Part*> parts = parts_at_tick(pos[0], instrument_map[index].tracks);
@@ -1105,8 +1113,13 @@ void DrumCanvas::keyPressed(int index, int velocity)
           if (parts.count() != 1)
             QMessageBox::warning(this, tr("Recording event failed"), tr("Couldn't record the event, because the currently selected part isn't the same track, and the instrument to be recorded could be either on no or on multiple parts, which is ambiguous.\nSelect the destination part, then try again."));
           else
-            steprec->record(*parts.begin(), instrument_map[index].pitch,ourDrumMap[index].len,editor->raster(),velocity,MusEGlobal::globalKeyState&Qt::ControlModifier,MusEGlobal::globalKeyState&Qt::ShiftModifier, -1 /* invalid pitch as "really played" -> the "insert rest" feature is never triggered */);
-          
+            steprec->record(
+              *parts.begin(),
+              instrument_map[index].pitch,ourDrumMap[index].len,
+              editor->raster(),
+              velocity,
+              MusEGlobal::globalKeyState&Qt::ControlModifier,
+              MusEGlobal::globalKeyState&Qt::ShiftModifier);
         }
       }            
 }
@@ -1236,7 +1249,7 @@ void DrumCanvas::resizeEvent(QResizeEvent* ev)
 
 void DrumCanvas::modifySelected(NoteInfo::ValType type, int val, bool delta_mode)
       {
-      QList< QPair<int,MusECore::Event> > already_done;
+      QList< QPair<QUuid,MusECore::Event> > already_done;
       MusECore::Undo operations;
       for (iCItem i = items.begin(); i != items.end(); ++i) {
             if (!(i->second->isSelected()))
@@ -1248,7 +1261,7 @@ void DrumCanvas::modifySelected(NoteInfo::ValType type, int val, bool delta_mode
 
             MusECore::MidiPart* part = (MusECore::MidiPart*)(e->part());
 
-            if (already_done.contains(QPair<int,MusECore::Event>(part->clonemaster_sn(), event)))
+            if (already_done.contains(QPair<QUuid,MusECore::Event>(part->clonemaster_uuid(), event)))
               continue;
             
             MusECore::Event newEvent = event.clone();
@@ -1324,7 +1337,7 @@ void DrumCanvas::modifySelected(NoteInfo::ValType type, int val, bool delta_mode
 
             operations.push_back(MusECore::UndoOp(MusECore::UndoOp::ModifyEvent, newEvent, event, part, false, false));
 
-            already_done.append(QPair<int,MusECore::Event>(part->clonemaster_sn(), event));
+            already_done.append(QPair<QUuid,MusECore::Event>(part->clonemaster_uuid(), event));
             }
       MusEGlobal::song->applyOperationGroup(operations);
       }
@@ -1546,10 +1559,17 @@ void DrumCanvas::midiNote(int pitch, int velo)
 
       if (_midiin && _steprec && !MusEGlobal::audio->isPlaying() && velo && !(MusEGlobal::globalKeyState & Qt::AltModifier) /* && pos[0] >= start_tick && pos[0] < end_tick [removed by flo93: this is handled in steprec->record()] */ )
       {
-         if (pitch == MusEGlobal::rcSteprecNote) // skip the fancy code below, simply record a rest
+         if (pitch == -1) // skip the fancy code below, simply record a rest
          {
            if (curPart)
-             steprec->record(curPart,0xdead,0xbeef,editor->raster(),velo,MusEGlobal::globalKeyState&Qt::ControlModifier,MusEGlobal::globalKeyState&Qt::ShiftModifier, pitch);
+             steprec->record(
+               curPart,
+               -1,
+               0,
+               editor->raster(),
+               velo,
+               MusEGlobal::globalKeyState&Qt::ControlModifier,
+               MusEGlobal::globalKeyState&Qt::ShiftModifier);
          }
          else
          {
@@ -1599,8 +1619,15 @@ void DrumCanvas::midiNote(int pitch, int velo)
             }
             
             if (rec_part != nullptr)
-              steprec->record(rec_part,instrument_map[rec_index].pitch,ourDrumMap[rec_index].len,editor->raster(),velo,MusEGlobal::globalKeyState&Qt::ControlModifier,MusEGlobal::globalKeyState&Qt::ShiftModifier, pitch);
-         }   
+              steprec->record(
+                rec_part,
+                instrument_map[rec_index].pitch,
+                ourDrumMap[rec_index].len,
+                editor->raster(),
+                velo,
+                MusEGlobal::globalKeyState&Qt::ControlModifier,
+                MusEGlobal::globalKeyState&Qt::ShiftModifier);
+         }
       }
 }
 
