@@ -353,10 +353,10 @@ DrumEdit::DrumEdit(MusECore::PartList* pl, QWidget* parent, const char* name, un
       menuScripts = menuBar()->addMenu(tr("&Scripts"));
       menuScripts->menuAction()->setStatusTip(tr("Scripts are applied to all or selected events. User scripts can be added in '~/.config/MusE/MusE/scripts/'. See 'MIDI scripting' in MusE wiki."));
       scripts.populateScriptMenu(menuScripts);
-      connect(&scripts,
+      _deliveredScriptReceivedMetaConn = connect(&scripts,
               &MusECore::Scripts::execDeliveredScriptReceived,
               [this](int id) { execDeliveredScript(id); } );
-      connect(&scripts,
+      _userScriptReceivedMetaConn = connect(&scripts,
               &MusECore::Scripts::execUserScriptReceived,
               [this](int id) { execUserScript(id); } );
 
@@ -605,7 +605,7 @@ DrumEdit::DrumEdit(MusECore::PartList* pl, QWidget* parent, const char* name, un
       canvas->setCanvasTools(drumeditTools);
       canvas->setFocus();
       connect(canvas, SIGNAL(toolChanged(int)), tools2, SLOT(set(int)));
-      connect(MusEGlobal::muse, &MusE::configChanged, tools2, &EditToolBar::configChanged);
+      _configChangedTools2MetaConn = connect(MusEGlobal::muse, &MusE::configChanged, tools2, &EditToolBar::configChanged);
       connect(canvas, SIGNAL(horizontalZoom(bool,const QPoint&)), SLOT(horizontalZoom(bool, const QPoint&)));
       connect(canvas, SIGNAL(horizontalZoom(int, const QPoint&)), SLOT(horizontalZoom(int, const QPoint&)));
       connect(canvas, SIGNAL(ourDrumMapChanged(bool)), SLOT(ourDrumMapChanged(bool)));
@@ -778,6 +778,13 @@ DrumEdit::DrumEdit(MusECore::PartList* pl, QWidget* parent, const char* name, un
       }
       }
 
+DrumEdit::~DrumEdit()
+{
+  disconnect(_configChangedTools2MetaConn);
+  disconnect(_deliveredScriptReceivedMetaConn);
+  disconnect(_userScriptReceivedMetaConn);
+}
+
 //---------------------------------------------------------
 //   songChanged1
 //---------------------------------------------------------
@@ -791,6 +798,11 @@ void DrumEdit::songChanged1(MusECore::SongChangedStruct_t bits)
         if(bits & SC_TRACK_REMOVED)
           checkTrackInfoTrack();
 
+        songChanged(bits);
+
+        if (parts()->empty())
+          return;
+
         if (bits & SC_DIVISION_CHANGED)
         {
           // The division has changed. The raster table and raster model will have been
@@ -809,18 +821,16 @@ void DrumEdit::songChanged1(MusECore::SongChangedStruct_t bits)
           setupHZoomRange();
         }
 
+        if ( ( bits & (SC_DRUMMAP | SC_TRACK_INSERTED | SC_TRACK_REMOVED | SC_TRACK_MODIFIED |
+                       SC_PART_INSERTED | SC_PART_REMOVED | SC_PART_MODIFIED) ) )
+          ((DrumCanvas*)(canvas))->rebuildOurDrumMap();
+
         if (bits & SC_SOLO)
         {
             if(canvas->track())
               toolbar->setSolo(canvas->track()->solo());
         }
 
-        if ( ( bits & (SC_DRUMMAP | SC_TRACK_INSERTED | SC_TRACK_REMOVED | SC_TRACK_MODIFIED |
-                       SC_PART_INSERTED | SC_PART_REMOVED | SC_PART_MODIFIED) ) )
-          ((DrumCanvas*)(canvas))->rebuildOurDrumMap();
-
-        songChanged(bits);
-
         // We'll receive SC_SELECTION if a different part is selected.
         // Addition - also need to respond here to moving part to another track. (Tim)
         if (bits & (SC_PART_INSERTED | SC_PART_REMOVED))
@@ -846,6 +856,10 @@ void DrumEdit::midiNote(int pitch, int velo)
 
     //      *note = old_style_drummap_mode ? ourDrumMap[index].anote : instrument_map[index].pitch;
 
+    // Ignore invalid pitches such as rest notes.
+    if(pitch < 0)
+      return;
+
     if ((DrumCanvas*)(canvas)->midiin())
     {
         int index=0;
