@@ -22,10 +22,9 @@
 //=========================================================
 
 #include <QToolTip>
+//#include <QDebug>
 
-#include <stdio.h>
-
-#include <map>
+//#include <stdio.h>
 
 #include "piano.h"
 #include "globals.h"
@@ -33,16 +32,16 @@
 #include "midiport.h"
 #include "track.h"
 #include "midictrl.h"
-#include "icons.h"
+//#include "icons.h"
 #include "utils.h"
 #include "gconfig.h"
 
 // Forwards from header:
 #include <QEvent>
 #include <QMouseEvent>
-#include <QWheelEvent>
+//#include <QWheelEvent>
 #include <QPainter>
-#include <QPixmap>
+//#include <QPixmap>
 #include "midieditor.h"
 
 namespace MusEGui {
@@ -194,6 +193,7 @@ Piano::Piano(QWidget* parent, int ymag, int width, MidiEditor* editor)
       pianoWidth(width)
 {
     setMouseTracking(true);
+    setFocusPolicy(Qt::ClickFocus);
     _midiEditor = editor;
     curPitch = -1;
     selectedPitch = 60;  // Start with 'C3"
@@ -412,15 +412,18 @@ void Piano::draw(QPainter& p, const QRect&, const QRegion&)
 //---------------------------------------------------------
 
 int Piano::pitch2y(int pitch) const
-      {
-      int tt[] = {
-            12, 19, 25, 32, 38, 51, 58, 64, 71, 77, 84, 90
-            };
-      int y = (75 * KH) - (tt[pitch%12] + (7 * KH) * (pitch/12));
-      if (y < 0)
-            y = 0;
-      return y;
-      }
+{
+	if (pitch<0)
+		return 0;
+
+	int tt[] = {
+		12, 19, 25, 32, 38, 51, 58, 64, 71, 77, 84, 90
+	};
+	int y = (75 * KH) - (tt[pitch%12] + (7 * KH) * (pitch/12));
+	if (y < 0)
+		y = 0;
+	return y;
+}
 
 //---------------------------------------------------------
 //   y2pitch
@@ -493,7 +496,7 @@ void Piano::viewMouseMoveEvent(QMouseEvent* event)
         if (nk < 0 || nk > 127)
             nk = -1;
         if (nk != keyDown) {
-            if (keyDown != -1) {
+            if (keyDown != -1 && !shift) {
                 emit keyReleased(keyDown, shift);
             }
             keyDown = nk;
@@ -514,8 +517,6 @@ void Piano::viewMouseMoveEvent(QMouseEvent* event)
 
     int v = qMax(1, qMin(127, (event->x() + 1) * 127 / pianoWidth));
     QString str = tr("Velocity: ") + QString::number(v);
-//    QString str = tr("Left click: Play") + " (Vel: " + QString::number(v) + ")\n"
-//            + tr("Right click: Select key");
     QToolTip::showText(event->globalPos(), str);
 }
 
@@ -529,7 +530,7 @@ void Piano::viewMousePressEvent(QMouseEvent* event)
     shift  = event->modifiers() & Qt::ShiftModifier;
 
     if (button == Qt::LeftButton) {
-        if (keyDown != -1) {
+        if (keyDown != -1 && !shift) {
             emit keyReleased(keyDown, shift);
             keyDown = -1;
         }
@@ -548,6 +549,11 @@ void Piano::viewMousePressEvent(QMouseEvent* event)
     }
 
     if (button == Qt::RightButton) {
+        // avoid stuck notes when LMB is pressed too
+        if (keyDown != -1 && !shift) {
+            emit keyReleased(keyDown, shift);
+            keyDown = -1;
+        }
         selectedPitch = y2pitch(event->y());
         emit curSelectedPitchChanged(selectedPitch);
         redraw();
@@ -565,7 +571,7 @@ void Piano::viewMouseReleaseEvent(QMouseEvent* event)
 {
     if (button == Qt::LeftButton) {
         shift = event->modifiers() & Qt::ShiftModifier;
-        if (keyDown != -1) {
+        if (keyDown != -1 && !shift) {
             emit keyReleased(keyDown, shift);
             keyDown = -1;
         }
@@ -575,6 +581,15 @@ void Piano::viewMouseReleaseEvent(QMouseEvent* event)
     button = Qt::NoButton;
 }
 
+void Piano::keyReleaseEvent(QKeyEvent *event) {
+    if (keyDown != -1 && event->key() == Qt::Key_Shift) {
+        emit shiftReleased();
+        keyDown = -1;
+    }
+    else
+        View::keyReleaseEvent(event);
+}
+
 //---------------------------------------------------------
 //   setCurSelectedPitch
 //---------------------------------------------------------
