@@ -26,17 +26,12 @@
 #include <QList>
 #include <QPair>
 #include <QToolTip>
+#include <QUuid>
 
-//#include <QDebug>
-
-//#include <limits.h>
 #include <stdio.h>
 #include "muse_math.h"
-//#include <errno.h>
 
 #include "prcanvas.h"
-//#include "view.h"
-//#include "mpevent.h"
 #include "globals.h"
 #include "cmd.h"
 #include "song.h"
@@ -48,17 +43,11 @@
 #include "type_defs.h"
 
 // Forwards from header:
-//#include <QDragEnterEvent>
-//#include <QDropEvent>
-//#include <QMouseEvent>
 #include <QDragMoveEvent>
-//#include <QDragLeaveEvent>
-//#include <QResizeEvent>
 #include <QPainter>
 #include <QWidget>
 #include "part.h"
 #include "event.h"
-//#include "pianoroll.h"
 #include "midieditor.h"
 #include "citem.h"
 #include "scrollscale.h"
@@ -1256,7 +1245,14 @@ void PianoCanvas::pianoPressed(int pitch, int velocity, bool shift)
       }
       
       if (_steprec && curPart) // && pos[0] >= start_tick && pos[0] < end_tick [removed by flo93: this is handled in steprec->record]
-          steprec->record(curPart,pitch,editor->raster(),editor->raster(),velocity,MusEGlobal::globalKeyState&Qt::ControlModifier,shift, -1 /* anything which is != rcSteprecNote */);
+          steprec->record(
+            curPart,
+            pitch,
+            editor->raster(),
+            editor->raster(),
+            velocity,
+            MusEGlobal::globalKeyState&Qt::ControlModifier,
+            shift);
       }
 
 //---------------------------------------------------------
@@ -1598,25 +1594,36 @@ void PianoCanvas::midiNote(int pitch, int velo)
       {
       if (MusEGlobal::debugMsg) printf("PianoCanvas::midiNote: pitch=%i, velo=%i\n", pitch, velo);
 
-      if (velo)
-        noteHeldDown[pitch]=true;
-      else
-        noteHeldDown[pitch]=false;
-
-      if (MusEGlobal::heavyDebugMsg)
+      // Ignore invalid pitches such as rest notes.
+      if(pitch >= 0)
       {
-        printf("  held down notes are: ");
-        for (int i=0;i<128;i++)
-          if (noteHeldDown[i])
-            printf("%i ",i);
-        printf("\n");
+        if (velo)
+          noteHeldDown[pitch]=true;
+        else
+          noteHeldDown[pitch]=false;
+
+        if (MusEGlobal::heavyDebugMsg)
+        {
+          printf("  held down notes are: ");
+          for (int i=0;i<128;i++)
+            if (noteHeldDown[i])
+              printf("%i ",i);
+          printf("\n");
+        }
       }
 
       if (_midiin && _steprec && curPart
          && !MusEGlobal::audio->isPlaying() && velo && pos[0] >= start_tick
          /* && pos[0] < end_tick [removed by flo93: this is handled in steprec->record] */
          && !(MusEGlobal::globalKeyState & Qt::AltModifier)) {
-                     steprec->record(curPart,pitch,editor->raster(),editor->raster(),velo,MusEGlobal::globalKeyState&Qt::ControlModifier,MusEGlobal::globalKeyState&Qt::ShiftModifier);
+                     steprec->record(
+                       curPart,
+                       pitch,
+                       editor->raster(),
+                       editor->raster(),
+                       velo,
+                       MusEGlobal::globalKeyState&Qt::ControlModifier,
+                       MusEGlobal::globalKeyState&Qt::ShiftModifier);
          }
       }
 
@@ -1767,7 +1774,7 @@ void PianoCanvas::curPartChanged()
 
 void PianoCanvas::modifySelected(MusEGui::NoteInfo::ValType type, int val, bool delta_mode)
 {
-    QList< QPair<int,MusECore::Event> > already_done;
+    QList< QPair<QUuid,MusECore::Event> > already_done;
     MusECore::Undo operations;
     unsigned int playedEventTick = UINT_MAX;
 
@@ -1781,7 +1788,7 @@ void PianoCanvas::modifySelected(MusEGui::NoteInfo::ValType type, int val, bool
 
         MusECore::MidiPart* part = (MusECore::MidiPart*)(e->part());
 
-        if (already_done.contains(QPair<int,MusECore::Event>(part->clonemaster_sn(), event)))
+        if (already_done.contains(QPair<QUuid,MusECore::Event>(part->clonemaster_uuid(), event)))
             continue;
 
         MusECore::Event newEvent = event.clone();
@@ -1870,9 +1877,7 @@ void PianoCanvas::modifySelected(MusEGui::NoteInfo::ValType type, int val, bool
         }
 
         operations.push_back(MusECore::UndoOp(MusECore::UndoOp::ModifyEvent, newEvent, event, part, false, false));
-        already_done.append(QPair<int,MusECore::Event>(part->clonemaster_sn(), event));
-
-        //setLastEdited(newEvent);
+        already_done.append(QPair<QUuid,MusECore::Event>(part->clonemaster_uuid(), event));
     }
     MusEGlobal::song->applyOperationGroup(operations);
 }
