@@ -77,6 +77,9 @@ class MidiComponentRack : public ComponentRack
         mStripComprProperty
       };
       
+      // Translate the component property id to a non controller id.
+      static MusECore::NonControllerId property2NonControllerId(MStripComponentProperties id);
+
   protected:
     MusECore::MidiTrack* _track;
     
@@ -310,12 +313,7 @@ class MidiStrip : public Strip {
       bool _isExpanded;
       // Current local state of knobs versus sliders preference global setting.
       bool _preferKnobs;
-      // Current local state of midi volume as decibels preference.
-      bool _preferMidiVolumeDb;
-
       int _heartBeatCounter;
-
-      double volume;
       bool inHeartBeat;
 
       void updateControls();
@@ -354,12 +352,14 @@ class MidiStrip : public Strip {
 
    public:
       MidiStrip(QWidget* parent, MusECore::MidiTrack*, bool hasHandle = false, bool isEmbedded = true, bool isDocked = false);
-      
+      virtual ~MidiStrip();
+
       static const double volSliderStepLin;
 
       static const double volSliderStepDb;
       static const double volSliderMaxDb;
       static const int    volSliderPrecDb;
+      static const QString volDBSymbol;
 
       static const int xMarginHorSlider;
       static const int yMarginHorSlider;
