@@ -69,6 +69,7 @@
 #include "meter.h"
 
 // For debugging output: Uncomment the fprintf section.
+//#include <stdio.h>
 #define DEBUG_STRIP(dev, format, args...) // fprintf(dev, format, ##args);
 
 using MusECore::UndoOp;
@@ -491,12 +492,11 @@ double ComponentRack::componentMaxValue(const ComponentWidget& cw) const
 }
 
 void ComponentRack::setComponentRange(const ComponentWidget& cw, double min, double max, bool updateOnly,
-                                      double step, int pageSize, 
-                                      DoubleRange::ConversionMode mode)
+                                      double step, int pageSize)
 {
   if(!cw._widget)
     return;
-  
+
   switch(cw._widgetType)
   {
     case CompactSliderComponentWidget:
@@ -507,7 +507,7 @@ void ComponentRack::setComponentRange(const ComponentWidget& cw, double min, dou
         if(updateOnly)
           w->blockSignals(true);
         if(min != w->minValue() && max != w->maxValue())
-          w->setRange(min, max, step, pageSize, mode);
+          w->setRange(min, max, step, pageSize);
         else if(min != w->minValue())
           w->setMinValue(max);
         else
@@ -526,7 +526,7 @@ void ComponentRack::setComponentRange(const ComponentWidget& cw, double min, dou
         if(updateOnly)
           w->blockSignals(true);
         if(min != w->minValue() && max != w->maxValue())
-          w->setRange(min, max, step, pageSize, mode);
+          w->setRange(min, max, step, pageSize);
         else if(min != w->minValue())
           w->setMinValue(max);
         else
@@ -851,14 +851,6 @@ TrackNameLabel::TrackNameLabel(QWidget* parent)
     _hovered = false;
 }
 
-//TrackNameLabel::TrackNameLabel(const QString& text, QWidget* parent, const char* name, Qt::WindowFlags f)
-// : QLabel(text, parent, name, f)
-//{
-//    _style3d = true;
-//    _hasExpandIcon = false;
-//    _expandIconPressed = false;
-//}
-
 void TrackNameLabel::mouseDoubleClickEvent(QMouseEvent* ev)
 {
   ev->accept();
@@ -884,10 +876,7 @@ void TrackNameLabel::paintEvent(QPaintEvent* ev)
   p.save();
     
   const QRect r = rect();
-
   p.fillRect(r.x(), r.y(), _expandIconWidth, r.height(), palette().mid());
-
-  //expandLeftRightSVGIcon->paint(&p, r.x() + r.width() - _expandIconWidth, r.y(), _expandIconWidth, r.height());
   expandLeftRightSVGIcon->paint(&p, r.x(), r.y(), _expandIconWidth, r.height());
   
   p.restore();
@@ -895,6 +884,10 @@ void TrackNameLabel::paintEvent(QPaintEvent* ev)
 
 void TrackNameLabel::mousePressEvent(QMouseEvent* ev)
 {
+  // Only one button at a time.
+  if(ev->buttons() ^ ev->button())
+    return;
+
   if (hasExpandIcon() && _hovered && (ev->pos().x() < _expandIconWidth))
   {
     _expandIconPressed = true;
@@ -904,6 +897,7 @@ void TrackNameLabel::mousePressEvent(QMouseEvent* ev)
   else
   {
     ev->ignore();
+    emit labelPressed(ev);
     QLabel::mousePressEvent(ev);
   }
 }
@@ -918,6 +912,7 @@ void TrackNameLabel::mouseReleaseEvent(QMouseEvent* ev)
   else
   {
     ev->ignore();
+    emit labelReleased(ev);
     QLabel::mouseReleaseEvent(ev);
   }
 }
@@ -931,6 +926,7 @@ void TrackNameLabel::mouseMoveEvent(QMouseEvent* ev)
     else
     {
         ev->ignore();
+        emit labelMoved(ev);
         QLabel::mouseMoveEvent(ev);
     }
 }
@@ -962,7 +958,6 @@ void TrackNameLabel::enterEvent(QEvent *e)
 //   Strip
 //---------------------------------------------------------
 
-//const int Strip::FIXED_METER_WIDTH = 7;
 
 //---------------------------------------------------------
 //   setRecordFlag
@@ -1077,6 +1072,67 @@ void Strip::trackNameLabelExpandClicked()
   setExpanded(!isExpanded());
 }
 
+void Strip::trackNameLabelPressed(QMouseEvent* ev)
+{
+  ev->accept();
+
+  const QPoint mousePos = QCursor::pos();
+  mouseWidgetOffset = pos() - mousePos;
+
+  if (ev->button() == Qt::LeftButton)
+  {
+    if(!_isEmbedded)
+    {
+      if (ev->modifiers() & Qt::ControlModifier)
+      {
+        setSelected(!isSelected());
+        track->setSelected(isSelected());
+        MusEGlobal::song->update(SC_TRACK_SELECTION);
+      }
+      else
+      {
+        emit clearStripSelection();
+        MusEGlobal::song->selectAllTracks(false);
+        setSelected(true);
+        track->setSelected(true);
+        MusEGlobal::song->update(SC_TRACK_SELECTION);
+      }
+    }
+  }
+}
+
+void Strip::trackNameLabelMoved(QMouseEvent* ev)
+{
+  ev->accept();
+
+  if(ev->buttons() == Qt::LeftButton)
+  {
+    if(!_isEmbedded)
+    {
+      if(dragOn)
+      {
+        QPoint mousePos = QCursor::pos();
+        move(mousePos + mouseWidgetOffset);
+      }
+      else
+      {
+        raise();
+        dragOn = true;
+      }
+    }
+  }
+}
+
+void Strip::trackNameLabelReleased(QMouseEvent* ev)
+{
+  ev->accept();
+
+  if (!_isEmbedded && dragOn) {
+    emit moveStrip(this);
+  }
+  dragOn=false;
+}
+
 //---------------------------------------------------------
 //   heartBeat
 //---------------------------------------------------------
@@ -1162,6 +1218,10 @@ void Strip::setLabelText()
 
 void Strip::muteToggled(bool val)
       {
+      // We receive toggled only if the button was checkable.
+      // If it was not checkable, pressed() and released() will handle it.
+      if(!mute || !mute->isCheckable())
+        return;
       if(track)
       {
         // This is a minor operation easily manually undoable. Let's not clog the undo list with it.
@@ -1178,13 +1238,20 @@ void Strip::muteToggled(bool val)
 
 void Strip::soloToggled(bool val)
 {
+    // We receive toggled only if the button was checkable.
+    // If it was not checkable, pressed() and released() will handle it.
+    if(!solo || !solo->isCheckable())
+      return;
     if (track && track->internalSolo()) {
         if (solo->isChecked())
             solo->setIcon(*soloAndProxyOnSVGIcon);
         else
             solo->setIcon(*soloProxyOnAloneSVGIcon);
     } else {
-        solo->setIcon(*soloStateSVGIcon);
+        if(solo->isDown())
+          solo->setIcon(*soloOnSVGIcon);
+        else
+          solo->setIcon(*soloOffSVGIcon);
     }
     //    solo->setIconSetB(track && track->internalSolo());
 
@@ -1196,6 +1263,251 @@ void Strip::soloToggled(bool val)
     MusEGlobal::audio->msgExecutePendingOperations(operations, true);
 }
 
+void Strip::mutePressed()
+{
+  // We receive toggled only if the button was checkable.
+  // If it was not checkable, pressed() and released() will handle it.
+  if(!mute || mute->isCheckable())
+    return;
+  if(track)
+  {
+    // This is a minor operation easily manually undoable. Let's not clog the undo list with it.
+    MusECore::PendingOperationList operations;
+    operations.add(MusECore::PendingOperationItem(track, true, MusECore::PendingOperationItem::SetTrackMute));
+    MusEGlobal::audio->msgExecutePendingOperations(operations, true);
+  }
+  updateMuteIcon();
+}
+
+void Strip::soloPressed()
+{
+  // We receive toggled only if the button was checkable.
+  // If it was not checkable, pressed() and released() will handle it.
+  if(!solo || solo->isCheckable())
+    return;
+  if (track && track->internalSolo()) {
+          solo->setIcon(*soloAndProxyOnSVGIcon);
+  } else {
+          solo->setIcon(*soloOnSVGIcon);
+  }
+
+  if (!track)
+      return;
+  // This is a minor operation easily manually undoable. Let's not clog the undo list with it.
+  MusECore::PendingOperationList operations;
+  operations.add(MusECore::PendingOperationItem(track, true, MusECore::PendingOperationItem::SetTrackSolo));
+  MusEGlobal::audio->msgExecutePendingOperations(operations, true);
+}
+
+void Strip::muteReleased()
+{
+  // We receive toggled only if the button was checkable.
+  // If it was not checkable, pressed() and released() will handle it.
+  if(!mute || mute->isCheckable())
+    return;
+  if(track)
+  {
+    // This is a minor operation easily manually undoable. Let's not clog the undo list with it.
+    MusECore::PendingOperationList operations;
+    operations.add(MusECore::PendingOperationItem(track, false, MusECore::PendingOperationItem::SetTrackMute));
+    MusEGlobal::audio->msgExecutePendingOperations(operations, true);
+  }
+}
+
+void Strip::soloReleased()
+{
+  // We receive toggled only if the button was checkable.
+  // If it was not checkable, pressed() and released() will handle it.
+  if(!solo || solo->isCheckable())
+    return;
+  if (track && track->internalSolo()) {
+          solo->setIcon(*soloProxyOnAloneSVGIcon);
+  } else {
+          solo->setIcon(*soloOffSVGIcon);
+  }
+
+  if (!track)
+      return;
+  // This is a minor operation easily manually undoable. Let's not clog the undo list with it.
+  MusECore::PendingOperationList operations;
+  operations.add(MusECore::PendingOperationItem(track, false, MusECore::PendingOperationItem::SetTrackSolo));
+  MusEGlobal::audio->msgExecutePendingOperations(operations, true);
+}
+
+void Strip::soloContextMenuReq(const QPoint& /*p*/) const
+{
+  MusEGlobal::song->execAutomationCtlPopup(track, QCursor::pos(), MusECore::MidiAudioCtrlStruct::NonAudioControl, MusECore::NCTL_TRACK_SOLO);
+}
+
+void Strip::muteContextMenuReq(const QPoint& /*p*/) const
+{
+  MusEGlobal::song->execAutomationCtlPopup(track, QCursor::pos(), MusECore::MidiAudioCtrlStruct::NonAudioControl, MusECore::NCTL_TRACK_MUTE);
+}
+
+void Strip::labelContextMenuReq(const QPoint& /*p*/)
+{
+  QMenu* menu = new QMenu;
+  QAction* act = nullptr;
+
+  // Only show these items if the strip is embedded outside the mixer,
+  //  since we now have mixer menu items for these.
+  if(isEmbedded())
+  {
+    menu->addAction(new MenuTitleItem(tr("Configuration"), menu));
+
+    act = menu->addAction(tr("Prefer Knobs, Not Sliders"));
+    act->setData(int(2));
+    act->setCheckable(true);
+    act->setChecked(MusEGlobal::config.preferKnobsVsSliders);
+
+    act = menu->addAction(tr("Show Values in Controls"));
+    act->setData(int(3));
+    act->setCheckable(true);
+    act->setChecked(MusEGlobal::config.showControlValues);
+
+    act = menu->addAction(tr("Prefer Midi Volume As Decibels"));
+    act->setData(int(4));
+    act->setCheckable(true);
+    act->setChecked(MusEGlobal::config.preferMidiVolumeDb);
+
+    menu->addSeparator();
+
+    act = menu->addAction(tr("Monitor on Record-arm Automatically"));
+    act->setData(int(5));
+    act->setCheckable(true);
+    act->setChecked(MusEGlobal::config.monitorOnRecord);
+
+    act = menu->addAction(tr("Momentary Mute"));
+    act->setData(int(6));
+    act->setCheckable(true);
+    act->setChecked(MusEGlobal::config.momentaryMute);
+
+    act = menu->addAction(tr("Momentary Solo"));
+    act->setData(int(7));
+    act->setCheckable(true);
+    act->setChecked(MusEGlobal::config.momentarySolo);
+
+    QMenu* audioEffectsRackVisibleItemsMenu = new QMenu(tr("Visible Audio Effects"));
+    QActionGroup* agroup = new QActionGroup(this);
+    agroup->setExclusive(true);
+    for(int i = 0; i <= MusECore::PipelineDepth; ++i)
+    {
+      act = audioEffectsRackVisibleItemsMenu->addAction(QString::number(i));
+      act->setData(int(5000 + i));
+      act->setCheckable(true);
+      act->setActionGroup(agroup);
+      if(i == MusEGlobal::config.audioEffectsRackVisibleItems)
+  //       act->setChecked(MusEGlobal::config.monitorOnRecord);
+        act->setChecked(true);
+    }
+    menu->addMenu(audioEffectsRackVisibleItemsMenu);
+  }
+
+  menu->addAction(new MenuTitleItem(tr("Actions"), menu));
+
+  act = menu->addAction(tr("Change Track Name"));
+  act->setData(int(1001));
+
+  if(!_isEmbedded)
+  {
+
+//      act = menu->addAction(tr("Remove track"));
+//      act->setData(int(0));
+//      menu->addSeparator();
+    act = menu->addAction(tr("Hide Strip"));
+    act->setData(int(1));
+  }
+
+  QPoint pt = QCursor::pos();
+  act = menu->exec(pt, nullptr);
+  if (!act)
+  {
+    delete menu;
+    return;
+  }
+
+  DEBUG_STRIP("Menu finished, data returned %d\n", act->data().toInt());
+
+  const int sel = act->data().toInt();
+  const bool checked = act->isChecked();
+  delete menu;
+
+  switch(sel)
+  {
+    case 0:
+      //  DEBUG_STRIP(stderr, "Strip:: delete track\n");
+      //  MusEGlobal::song->applyOperation(UndoOp(UndoOp::DeleteTrack, MusEGlobal::song->tracks()->index(track), track));
+    break;
+
+    case 1:
+      DEBUG_STRIP(stderr, "Strip:: setStripVisible false \n");
+      setStripVisible(false);
+      setVisible(false);
+      emit visibleChanged(this, false);
+    break;
+
+    case 2:
+      if(MusEGlobal::config.preferKnobsVsSliders != checked)
+      {
+        MusEGlobal::config.preferKnobsVsSliders = checked;
+        MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
+      }
+    break;
+
+    case 3:
+      if(MusEGlobal::config.showControlValues != checked)
+      {
+        MusEGlobal::config.showControlValues = checked;
+        MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
+      }
+    break;
+
+    case 4:
+      if(MusEGlobal::config.preferMidiVolumeDb != checked)
+      {
+        MusEGlobal::config.preferMidiVolumeDb = checked;
+        MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
+      }
+    break;
+
+    case 5:
+      if(MusEGlobal::config.monitorOnRecord != checked)
+      {
+        MusEGlobal::config.monitorOnRecord = checked;
+        MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
+      }
+    break;
+
+    case 6:
+      if(MusEGlobal::config.momentaryMute != checked)
+      {
+        MusEGlobal::config.momentaryMute = checked;
+        MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
+      }
+    break;
+
+    case 7:
+      if(MusEGlobal::config.momentarySolo != checked)
+      {
+        MusEGlobal::config.momentarySolo = checked;
+        MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
+      }
+    break;
+
+    case 1001:
+      changeTrackName();
+    break;
+
+    default:
+      if(sel >= 5000 && sel <= 5000 + MusECore::PipelineDepth)
+      {
+        MusEGlobal::config.audioEffectsRackVisibleItems = sel - 5000;
+        MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
+      }
+    break;
+  }
+}
+
 //---------------------------------------------------------
 //   Strip
 //    create mixer strip
@@ -1274,6 +1586,7 @@ Strip::Strip(QWidget* parent, MusECore::Track* t, bool hasHandle, bool isEmbedde
       label->setContentsMargins(0, 0, 0, 0);
       label->setAlignment(Qt::AlignCenter);
       label->setAutoFillBackground(true);
+      label->setContextMenuPolicy(Qt::CustomContextMenu);
       label->ensurePolished();
       if (label->style3d()) {
           label->setLineWidth(2);
@@ -1292,6 +1605,11 @@ Strip::Strip(QWidget* parent, MusECore::Track* t, bool hasHandle, bool isEmbedde
 
       connect(label, &TrackNameLabel::doubleClicked, [this]() { changeTrackName(); } );
       connect(label, &TrackNameLabel::expandClicked, [this]() { trackNameLabelExpandClicked(); } );
+
+      connect(label, &TrackNameLabel::labelPressed, [this](QMouseEvent* ev) { trackNameLabelPressed(ev); } );
+      connect(label, &TrackNameLabel::labelMoved, [this](QMouseEvent* ev) { trackNameLabelMoved(ev); } );
+      connect(label, &TrackNameLabel::labelReleased, [this](QMouseEvent* ev) { trackNameLabelReleased(ev); } );
+      connect(label, &TrackNameLabel::customContextMenuRequested, [this](const QPoint &p) { labelContextMenuReq(p); } );
       }
 
 //---------------------------------------------------------
@@ -1388,171 +1706,6 @@ void Strip::updateRouteButtons()
         }
     }
 }
-
-void Strip::mousePressEvent(QMouseEvent* ev)
-{
-  ev->accept();
-
-  // Only one button at a time.
-  if(ev->buttons() ^ ev->button())
-    return;
-
-  QPoint mousePos = QCursor::pos();
-  mouseWidgetOffset = pos() - mousePos;
-
-  if (ev->button() == Qt::RightButton) {
-    QMenu* menu = new QMenu;
-
-    menu->addAction(new MenuTitleItem(tr("Configuration"), menu));
-
-    QAction* act = menu->addAction(tr("Prefer Knobs, Not Sliders"));
-    act->setData(int(2));
-    act->setCheckable(true);
-    act->setChecked(MusEGlobal::config.preferKnobsVsSliders);
-
-    act = menu->addAction(tr("Show Values in Controls"));
-    act->setData(int(3));
-    act->setCheckable(true);
-    act->setChecked(MusEGlobal::config.showControlValues);
-
-    act = menu->addAction(tr("Prefer Midi Volume As Decibels"));
-    act->setData(int(4));
-    act->setCheckable(true);
-    act->setChecked(MusEGlobal::config.preferMidiVolumeDb);
-
-    menu->addSeparator();
-
-    act = menu->addAction(tr("Monitor on Record-arm Automatically"));
-    act->setData(int(5));
-    act->setCheckable(true);
-    act->setChecked(MusEGlobal::config.monitorOnRecord);
-
-    QMenu* audioEffectsRackVisibleItemsMenu = new QMenu(tr("Visible Audio Effects"));
-    QActionGroup* agroup = new QActionGroup(this);
-    agroup->setExclusive(true);
-    for(int i = 0; i <= MusECore::PipelineDepth; ++i)
-    {
-      act = audioEffectsRackVisibleItemsMenu->addAction(QString::number(i));
-      act->setData(int(5000 + i));
-      act->setCheckable(true);
-      act->setActionGroup(agroup);
-      if(i == MusEGlobal::config.audioEffectsRackVisibleItems)
-        act->setChecked(MusEGlobal::config.monitorOnRecord);
-    }
-    menu->addMenu(audioEffectsRackVisibleItemsMenu);
-    
-    menu->addAction(new MenuTitleItem(tr("Actions"), menu));
-
-    act = menu->addAction(tr("Change Track Name"));
-    act->setData(int(1001));
-
-    if(!_isEmbedded)
-    {
-
-//      act = menu->addAction(tr("Remove track"));
-//      act->setData(int(0));
-//      menu->addSeparator();
-      act = menu->addAction(tr("Hide Strip"));
-      act->setData(int(1));
-    }
-
-    QPoint pt = QCursor::pos();
-    act = menu->exec(pt, nullptr);
-    if (!act)
-    {
-      delete menu;
-      return;
-    }
-
-    DEBUG_STRIP("Menu finished, data returned %d\n", act->data().toInt());
-
-    const int sel = act->data().toInt();
-    const bool checked = act->isChecked();
-    delete menu;
-
-    switch(sel)
-    {
-      case 0:
-        //  DEBUG_STRIP(stderr, "Strip:: delete track\n");
-        //  MusEGlobal::song->applyOperation(UndoOp(UndoOp::DeleteTrack, MusEGlobal::song->tracks()->index(track), track));
-      break;
-
-      case 1:
-        DEBUG_STRIP(stderr, "Strip:: setStripVisible false \n");
-        setStripVisible(false);
-        setVisible(false);
-        emit visibleChanged(this, false);
-      break;
-
-      case 2:
-        if(MusEGlobal::config.preferKnobsVsSliders != checked)
-        {
-          MusEGlobal::config.preferKnobsVsSliders = checked;
-          MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
-        }
-      break;
-
-      case 3:
-        if(MusEGlobal::config.showControlValues != checked)
-        {
-          MusEGlobal::config.showControlValues = checked;
-          MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
-        }
-      break;
-
-      case 4:
-        if(MusEGlobal::config.preferMidiVolumeDb != checked)
-        {
-          MusEGlobal::config.preferMidiVolumeDb = checked;
-          MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
-        }
-      break;
-
-      case 5:
-        if(MusEGlobal::config.monitorOnRecord != checked)
-        {
-          MusEGlobal::config.monitorOnRecord = checked;
-          MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
-        }
-      break;
-
-      case 1001:
-        changeTrackName();
-      break;
-      
-      default:
-        if(sel >= 5000 && sel <= 5000 + MusECore::PipelineDepth)
-        {
-          MusEGlobal::config.audioEffectsRackVisibleItems = sel - 5000;
-          MusEGlobal::muse->changeConfig(true); // Save settings immediately, and use simple version.
-        }
-      break;
-    }
-
-    ev->accept();
-    return;
-  }
-  else if (ev->button() == Qt::LeftButton)
-  {
-    if(!_isEmbedded)
-    {
-      if (ev->modifiers() & Qt::ControlModifier)
-      {
-        setSelected(!isSelected());
-        track->setSelected(isSelected());
-        MusEGlobal::song->update(SC_TRACK_SELECTION);
-      }
-      else
-      {
-        emit clearStripSelection();
-        MusEGlobal::song->selectAllTracks(false);
-        setSelected(true);
-        track->setSelected(true);
-        MusEGlobal::song->update(SC_TRACK_SELECTION);
-      }
-    }
-  }
-}
  
 QSize Strip::sizeHint() const
 {
@@ -1786,36 +1939,6 @@ QSize ExpanderHandle::sizeHint() const
   return sz;
 }
 
-
-void Strip::mouseReleaseEvent(QMouseEvent* ev)
-{
-  ev->accept();
-  if (!_isEmbedded && dragOn) {
-    emit moveStrip(this);
-  }
-  dragOn=false;
-}
-
-void Strip::mouseMoveEvent(QMouseEvent* e)
-{
-  e->accept();
-  if(e->buttons() == Qt::LeftButton)
-  {
-    if(!_isEmbedded)
-    {
-      if(dragOn)
-      {
-        QPoint mousePos = QCursor::pos();
-        move(mousePos + mouseWidgetOffset);
-      }
-      else
-      {
-        raise();
-        dragOn = true;
-      }
-    }
-  }
-}
 bool Strip::handleForwardedKeyPress(QKeyEvent* event)
 {  
   const int kb_code = event->key() | event->modifiers();
@@ -1854,12 +1977,18 @@ bool Strip::handleForwardedKeyPress(QKeyEvent* event)
   }
   else if (kb_code ==  MusEGui::shortcuts[MusEGui::SHRT_MUTE_CURRENT_TRACKS].key)
   {
-      mute->setChecked(!mute->isChecked());
+      // Only if not momentary.
+      if(mute->isCheckable())
+        // This will emit toggled, telling the mute to toggle.
+        mute->setChecked(!mute->isChecked());
       return true;
   }
   else if (kb_code == MusEGui::shortcuts[MusEGui::SHRT_SOLO_CURRENT_TRACKS].key)
   {
-      solo->setChecked(!solo->isChecked());
+      // Only if not momentary.
+      if(solo->isCheckable())
+        // This will emit toggled, telling the solo to toggle.
+        solo->setChecked(!solo->isChecked());
       return true;
   }
   return false;
@@ -1948,12 +2077,15 @@ void Strip::updateMuteIcon()
     }
     //  mute->setIconSetB(found && !track->internalSolo() && !track->solo());
     if (found && !track->internalSolo() && !track->solo()) {
-        if (mute->isChecked())
+        if (mute->isDown())
             mute->setIcon(*muteAndProxyOnSVGIcon);
         else
             mute->setIcon(*muteProxyOnSVGIcon);
     } else {
-        mute->setIcon(*muteStateSVGIcon);
+        if(mute->isDown())
+          mute->setIcon(*muteOnSVGIcon);
+        else
+          mute->setIcon(*muteOffSVGIcon);
     }
 }
 
@@ -2026,24 +2158,13 @@ void Strip::componentChanged(int type, double val, bool off, int id, int scrollM
     double ma_val = val;
     if(id == MusECore::CTRL_VOLUME)
     {
-      if(MusEGlobal::config.preferMidiVolumeDb)
+      // It's a log scale. We need to convert to Db and
+      //  scale the linear value by 2 since the midi scale is
+      //  twice the usual Db per step. Then convert back to log.
+      if(m_val > 0)
       {
-        if(ma_val <= MusEGlobal::config.minSlider)
-          m_val = ma_val = 0.0;
-        else
-        {
-          m_val = double(i_m_max) * muse_db2val(m_val);
-          ma_val = double(i_m_max) * muse_db2val(ma_val / 2.0);
-        }
-      }
-      else
-      {
-        // It's a linear scale. We need to convert to Db and
-        //  scale the linear value by 2 since the midi scale is
-        //  twice the usual Db per step. Then convert back to linear.
-        m_val = muse_val2dbr(m_val / double(i_m_max)) * 2.0;
-        m_val *= 2.0;
-        m_val = double(i_m_max) * muse_db2val(m_val / 2.0);
+        m_val = museValToDb(m_val / double(i_m_max), 2 * 40.0);
+        m_val = double(i_m_max) * museDbToVal(m_val, 1 / 40.0);
       }
     }
 
@@ -2127,6 +2248,7 @@ void Strip::componentChanged(int type, double val, bool off, int id, int scrollM
         //  the top midi value of 127 represents 0dB. Cut it off at 0dB.
         const double a_min = cl->minVal();
         const double a_max = (a_ctlnum == MusECore::AC_VOLUME) ? 1.0 : cl->maxVal();
+
         const double a_range = a_max - a_min;
         const double a_val = (m_fact * a_range) + a_min;
 
@@ -2171,21 +2293,17 @@ void Strip::componentChanged(int type, double val, bool off, int id, int scrollM
     //----------------------------------------------------------
     double a_val = val;
     double ma_val = val;
-    if(id == MusECore::AC_VOLUME)
-    {
-      if(ma_val <= MusEGlobal::config.minSlider)
-        a_val = ma_val = 0.0;
-      else
-      {
-        a_val = muse_db2val(a_val);
-        ma_val = muse_db2val(ma_val / 2.0);
-      }
-    }
+    if(ma_val <= 0.0)
+      ma_val = MusEGlobal::config.minSlider;
+    else
+      ma_val = muse_val2db(ma_val);
+    ma_val = muse_db2val(ma_val / 2.0);
 
     // The audio volume can go above 0dB (amplification) while
     //  the top midi value of 127 represents 0dB. Cut it off at 0dB.
     const double a_min = cl->minVal();
     const double a_max = (id == MusECore::AC_VOLUME) ? 1.0 : cl->maxVal();
+
     const double a_range = a_max - a_min;
     if(a_range < 0.0001) // Avoid divide by zero.
       return;
@@ -2318,27 +2436,16 @@ void Strip::componentPressed(int type, double val, int id)
     //  does 10 ^ (volume dB / 20), just divide volume dB by 2.
     //----------------------------------------------------------
     double m_val = val;
-    double ma_val = val;
+//     double ma_val = val;
     if(id == MusECore::CTRL_VOLUME)
     {
-      if(MusEGlobal::config.preferMidiVolumeDb)
+      // It's a log scale. We need to convert to Db and
+      //  scale the linear value by 2 since the midi scale is
+      //  twice the usual Db per step. Then convert back to log.
+      if(m_val > 0)
       {
-        if(ma_val <= MusEGlobal::config.minSlider)
-          m_val = ma_val = 0.0;
-        else
-        {
-          m_val = double(i_m_max) * muse_db2val(m_val);
-          ma_val = double(i_m_max) * muse_db2val(ma_val / 2.0);
-        }
-      }
-      else
-      {
-        // It's a linear scale. We need to convert to Db and
-        //  scale the linear value by 2 since the midi scale is
-        //  twice the usual Db per step. Then convert back to linear.
-        m_val = muse_val2dbr(m_val / double(i_m_max)) * 2.0;
-        m_val *= 2.0;
-        m_val = double(i_m_max) * muse_db2val(m_val / 2.0);
+        m_val = museValToDb(m_val / double(i_m_max), 2 * 40.0);
+        m_val = double(i_m_max) * museDbToVal(m_val, 1 / 40.0);
       }
     }
 
@@ -2421,14 +2528,6 @@ void Strip::componentPressed(int type, double val, int id)
     }
 
     double a_val = val;
-    if(id == MusECore::AC_VOLUME)
-    {
-      if(a_val <= MusEGlobal::config.minSlider)
-        a_val = 0.0;
-      else
-        a_val = muse_db2val(a_val);
-    }
-
     MusECore::TrackList* tracks = MusEGlobal::song->tracks();
     for(MusECore::iTrack it = tracks->begin(); it != tracks->end(); ++it)
     {
@@ -2508,27 +2607,16 @@ void Strip::componentReleased(int type, double val, int id)
     //  does 10 ^ (volume dB / 20), just divide volume dB by 2.
     //----------------------------------------------------------
     double m_val = val;
-    double ma_val = val;
+//     double ma_val = val;
     if(id == MusECore::CTRL_VOLUME)
     {
-      if(MusEGlobal::config.preferMidiVolumeDb)
+      // It's a log scale. We need to convert to Db and
+      //  scale the linear value by 2 since the midi scale is
+      //  twice the usual Db per step. Then convert back to log.
+      if(m_val > 0)
       {
-        if(ma_val <= MusEGlobal::config.minSlider)
-          m_val = ma_val = 0.0;
-        else
-        {
-          m_val = double(i_m_max) * muse_db2val(m_val);
-          ma_val = double(i_m_max) * muse_db2val(ma_val / 2.0);
-        }
-      }
-      else
-      {
-        // It's a linear scale. We need to convert to Db and
-        //  scale the linear value by 2 since the midi scale is
-        //  twice the usual Db per step. Then convert back to linear.
-        m_val = muse_val2dbr(m_val / double(i_m_max)) * 2.0;
-        m_val *= 2.0;
-        m_val = double(i_m_max) * muse_db2val(m_val / 2.0);
+        m_val = museValToDb(m_val / double(i_m_max), 2 * 40.0);
+        m_val = double(i_m_max) * museDbToVal(m_val, 1 / 40.0);
       }
     }
 
@@ -2590,7 +2678,7 @@ void Strip::componentReleased(int type, double val, int id)
 
         MusECore::AutomationType atype = at->automationType();
         at->stopAutoRecord(a_ctlnum, a_val);
-        if(atype == MusECore::AUTO_OFF || atype == MusECore::AUTO_TOUCH)
+        if(atype == MusECore::AUTO_OFF || (atype == MusECore::AUTO_READ && MusEGlobal::audio->isPlaying()) ||atype == MusECore::AUTO_TOUCH)
           at->enableController(a_ctlnum, true);
       }
     }
@@ -2618,14 +2706,6 @@ void Strip::componentReleased(int type, double val, int id)
     //  does 10 ^ (volume dB / 20), just divide volume dB by 2.
     //----------------------------------------------------------
     double a_val = val;
-    if(id == MusECore::AC_VOLUME)
-    {
-      if(a_val <= MusEGlobal::config.minSlider)
-        a_val = 0.0;
-      else
-        a_val = muse_db2val(a_val);
-    }
-
     MusECore::TrackList* tracks = MusEGlobal::song->tracks();
     for(MusECore::iTrack it = tracks->begin(); it != tracks->end(); ++it)
     {
@@ -2741,11 +2821,11 @@ void Strip::componentIncremented(int type, double oldCompVal, double newCompVal,
         // It's a linear scale. We need to convert to Db and
         //  scale the linear value by 2 since the midi scale is
         //  twice the usual Db per step. Then convert back to linear.
-        ma_old_val = muse_val2dbr(ma_old_val / double(i_ma_max)) * 2.0;
+        ma_old_val = muse_val2db(ma_old_val / double(i_ma_max)) * 2.0;
         ma_old_val *= 2.0;
         ma_old_val = double(i_ma_max) * muse_db2val(ma_old_val / 2.0);
 
-        ma_new_val = muse_val2dbr(ma_new_val / double(i_ma_max)) * 2.0;
+        ma_new_val = muse_val2db(ma_new_val / double(i_ma_max)) * 2.0;
         ma_new_val *= 2.0;
         ma_new_val = double(i_ma_max) * muse_db2val(ma_new_val / 2.0);
       }
@@ -2846,7 +2926,7 @@ void Strip::componentIncremented(int type, double oldCompVal, double newCompVal,
             d_cur_val = d_cur_val - double(mctl->bias());
             if(id == MusECore::CTRL_VOLUME && MusEGlobal::config.preferMidiVolumeDb)
             {
-              d_fin_val = muse_val2dbr(d_cur_val / double(max)) * 2.0;
+              d_fin_val = muse_val2db(d_cur_val / double(max)) * 2.0;
               d_fin_val += d_comp_val_delta;
               d_fin_val = double(max) * muse_db2val(d_fin_val / 2.0);
             }
@@ -2894,7 +2974,7 @@ void Strip::componentIncremented(int type, double oldCompVal, double newCompVal,
           if(d_cur_val <= 0.0)
             d_cur_val = MusEGlobal::config.minSlider;
           else
-            d_cur_val = muse_val2dbr(d_cur_val);
+            d_cur_val = muse_val2db(d_cur_val);
 
           d_cur_val += d_comp_val_delta;
           if(d_cur_val < MusEGlobal::config.minSlider)
@@ -2951,18 +3031,12 @@ void Strip::componentIncremented(int type, double oldCompVal, double newCompVal,
     // Unusual, it is a factor of 40 not 20. Since muse_db2val()
     //  does 10 ^ (volume dB / 20), just divide volume dB by 2.
     //----------------------------------------------------------
-    double a_val = newCompVal;
     double ma_val = newCompVal;
-    if(id == MusECore::AC_VOLUME)
-    {
-      if(ma_val <= MusEGlobal::config.minSlider)
-        a_val = ma_val = 0.0;
-      else
-      {
-        a_val = muse_db2val(a_val);
-        ma_val = muse_db2val(ma_val / 2.0);
-      }
-    }
+    if(ma_val <= 0.0)
+      ma_val = MusEGlobal::config.minSlider;
+    else
+      ma_val = muse_val2db(ma_val);
+    ma_val = muse_db2val(ma_val / 2.0);
 
     // The audio volume can go above 0dB (amplification) while
     //  the top midi value of 127 represents 0dB. Cut it off at 0dB.
@@ -3035,7 +3109,7 @@ void Strip::componentIncremented(int type, double oldCompVal, double newCompVal,
             d_cur_val = d_cur_val - double(mctl->bias());
             if(m_ctlnum == MusECore::CTRL_VOLUME)
             {
-              d_fin_val = muse_val2dbr(d_cur_val / d_max) * 2.0;
+              d_fin_val = muse_val2db(d_cur_val / d_max) * 2.0;
               d_fin_val += d_comp_val_delta;
               d_fin_val = d_max * muse_db2val(d_fin_val / 2.0);
             }
@@ -3083,7 +3157,7 @@ void Strip::componentIncremented(int type, double oldCompVal, double newCompVal,
           if(d_cur_val <= 0.0)
             d_cur_val = MusEGlobal::config.minSlider;
           else
-            d_cur_val = muse_val2dbr(d_cur_val);
+            d_cur_val = muse_val2db(d_cur_val);
 
           d_cur_val += d_comp_val_delta;
           if(d_cur_val < MusEGlobal::config.minSlider)
