@@ -53,6 +53,7 @@ class MidiPort;
 class MetroAccentsMap;
 class AudioConverterSettingsGroup;
 class AudioConverterPluginI;
+class MidiRemote;
 
 typedef std::list < iMidiCtrlValList > MidiCtrlValListIterators_t;
 typedef MidiCtrlValListIterators_t::iterator iMidiCtrlValListIterators_t;
@@ -167,8 +168,12 @@ struct PendingOperationItem
     ModifyMidiCtrlValList,
 
     RemapDrumControllers,
-    AddAudioCtrlVal,   DeleteAudioCtrlVal,    ModifyAudioCtrlVal, ModifyAudioCtrlValList,
-    ModifyTempoList,   SetStaticTempo,        SetGlobalTempo, 
+    AddAudioCtrlVal,   AddAudioCtrlValStruct, DeleteAudioCtrlVal,    ModifyAudioCtrlVal, ModifyAudioCtrlValList,
+    AddAudioCtrlValList, DeleteAudioCtrlValList, ModifyAudioCtrlValListList,
+    SelectAudioCtrlVal, SetAudioCtrlPasteEraseMode,
+    UpdateAllAudioCtrlGroups, UpdateAudioCtrlListGroups, UpdateAudioCtrlGroups, UpdateAudioCtrlPosGroups,
+
+    ModifyTempoList,   SetStaticTempo,        SetGlobalTempo,
     ModifySigList,
     ModifyKeyList,
 
@@ -186,7 +191,8 @@ struct PendingOperationItem
     ModifyAudioSamples,
     SwitchMetronomeSettings, ModifyMetronomeAccentMap,
     SetExternalSyncFlag, SetUseJackTransport, SetUseMasterTrack,
-    ModifyMarkerList
+    ModifyMarkerList,
+    SwitchMidiRemoteSettings, ModifyMidiRemote
     }; 
                               
   PendingOperationType _type;
@@ -196,6 +202,7 @@ struct PendingOperationItem
     MidiPort* _midi_port;
     void* _void_track_list;
     int* _audioSamplesLen;
+    CtrlListList* _src_aud_ctrl_list_list;
   };
   
   union {
@@ -216,6 +223,7 @@ struct PendingOperationItem
     MarkerList** _orig_marker_list;
     float** _audioSamplesPointer;
     MetroAccentsMap** _metroAccentsMap;
+    MidiRemote* _midiRemote;
   };
             
   union {
@@ -234,6 +242,7 @@ struct PendingOperationItem
     bool* _bool_pointer;
     MetroAccentsMap* _newMetroAccentsMap;
     AudioConverterSettingsGroup* _audio_converter_settings;
+    MidiRemote* _newMidiRemote;
   };
 
   iPart _iPart; 
@@ -249,7 +258,7 @@ struct PendingOperationItem
   iMarker _iMarker;
   Route _src_route;
   Route _dst_route;
-  // SndFileR is only 8 bytes but can't be in a union becuase of non-trivial destructor (?).
+  // SndFileR is only 8 bytes but can't be in a union because of non-trivial destructor (?).
   SndFileR _sndFileR;
   
   union {
@@ -271,6 +280,7 @@ struct PendingOperationItem
     MidiCtrlValRemapOperation* _midi_ctrl_val_remap_operation;
     MuseFrame_t _museFrame;
     AudioConverterPluginI* _audio_converter;
+    CtrlList::PasteEraseOptions _audio_ctrl_paste_erase_opts;
   };
   
   union {
@@ -278,10 +288,12 @@ struct PendingOperationItem
     unsigned int _uintB;
     unsigned int _lenVal;
     unsigned int _marker_tick;
+    bool _selected;
     int _to_idx;
     int _address_port;
     int _open_flags;
     int _ctl_num;
+    CtrlVal::CtrlValueFlags _ctl_flags;
     int _stretch_type;
     AudioConverterPluginI* _audio_converter_ui;
   };
@@ -293,8 +305,21 @@ struct PendingOperationItem
     double _ctl_dbl_val;
     double _audio_converter_value;
     bool _marker_lock;
+    CtrlVal* _audCtrlValStruct;
   };
 
+  //========================================================================
+  //  NOTICE: Where these constructors take pointers to allocated objects,
+  //           some of them we do NOT take ownership of, while some we do.
+  //          It mostly depends on whether the undo system (which is by far
+  //           top user of this operations system) needs to keep them around
+  //           or whether the objects are temporary once-only things.
+  //          For example strings created by the undo system are kept and
+  //           put onto the undo stack so we don't take ownership of them,
+  //           while swap-out lists created by the undo system are temporary
+  //           so for convenience we DO take ownership of them.
+  //========================================================================
+
   PendingOperationItem(AudioConverterSettingsGroup* new_settings,
                        PendingOperationType type = ModifyDefaultAudioConverterSettings)
     { _type = type; _audio_converter_settings = new_settings; }
@@ -445,7 +470,7 @@ struct PendingOperationItem
   PendingOperationItem(Part* part, const Event& ev, PendingOperationType type = AddEvent)
     { _type = type; _part = part; _ev = ev; }
     
-  // NOTE: To avoid possibly deleting the event in RT stage 2 when the event is erased from the list, 
+  // NOTE: To avoid possibly deleting the event in RT stage 2 when the event is erased from the list,
   //        _ev is used simply to hold a reference until non-RT stage 3 or after, when the list is cleared.
   PendingOperationItem(Part* part, const iEvent& iev, PendingOperationType type = DeleteEvent)
     { _type = type; _part = part; _iev = iev; _ev = iev->second; }
@@ -460,7 +485,9 @@ struct PendingOperationItem
     
   PendingOperationItem(const iCtrlList& ictl_l, CtrlList* ctrl_l, PendingOperationType type = ModifyAudioCtrlValList)
     { _type = type; _iCtrlList = ictl_l; _aud_ctrl_list = ctrl_l; }
-    
+  PendingOperationItem(CtrlListList* dst, CtrlListList* src, PendingOperationType type = ModifyAudioCtrlValListList)
+    { _type = type; _aud_ctrl_list_list = dst; _src_aud_ctrl_list_list = src; }
+
   PendingOperationItem(MidiCtrlValList* mcvl, Part* part, unsigned int tick, int val, PendingOperationType type = AddMidiCtrlVal)
     { _type = type; _mcvl = mcvl; _part = part; 
         _posLenVal = tick; _intB = val; }
@@ -475,18 +502,39 @@ struct PendingOperationItem
     
   PendingOperationItem(MidiCtrlValList* orig_mcvl, MidiCtrlValList* mcvl, PendingOperationType type = ModifyMidiCtrlValList)
     { _type = type; _orig_mcvl = orig_mcvl; _mcvl = mcvl; }
-    
-  PendingOperationItem(CtrlList* ctrl_l, unsigned int frame, double ctrl_val, PendingOperationType type = AddAudioCtrlVal)
-    { _type = type; _aud_ctrl_list = ctrl_l; _posLenVal = frame; _ctl_dbl_val = ctrl_val; }
-    
-  PendingOperationItem(CtrlList* ctrl_l, const iCtrl& ictl, PendingOperationType type = DeleteAudioCtrlVal)
+
+
+  PendingOperationItem(CtrlListList* cll, CtrlList* cl, PendingOperationType type = AddAudioCtrlValList)
+    { _type = type; _aud_ctrl_list_list = cll; _aud_ctrl_list = cl; }
+  PendingOperationItem(CtrlListList* cll, const iCtrlList& icll, PendingOperationType type = DeleteAudioCtrlValList)
+    { _type = type; _aud_ctrl_list_list = cll; _iCtrlList = icll; }
+
+  PendingOperationItem(CtrlList* ctrl_l, unsigned int frame, double ctrl_val,
+                       CtrlVal::CtrlValueFlags flags, PendingOperationType type = AddAudioCtrlVal)
+    { _type = type; _aud_ctrl_list = ctrl_l; _posLenVal = frame; _ctl_dbl_val = ctrl_val; _ctl_flags = flags; }
+  PendingOperationItem(CtrlList* ctrl_l, unsigned int frame, CtrlVal* ctrl_v, PendingOperationType type = AddAudioCtrlValStruct)
+    { _type = type; _aud_ctrl_list = ctrl_l; _posLenVal = frame; _audCtrlValStruct = ctrl_v; }
+
+  // Type is DeleteAudioCtrlVal or UpdateAudioCtrlPosGroups.
+  PendingOperationItem(CtrlList* ctrl_l, const iCtrl& ictl, PendingOperationType type)
     { _type = type; _aud_ctrl_list = ctrl_l; _iCtrl = ictl; }
     
   // NOTE: ctrl_l is supplied in case the operation needs to be merged, or transformed into an AddAudioCtrlVal.
   PendingOperationItem(CtrlList* ctrl_l, const iCtrl& ictl, unsigned int new_frame,
                        double new_ctrl_val, PendingOperationType type = ModifyAudioCtrlVal)
     { _type = type; _aud_ctrl_list = ctrl_l; _iCtrl = ictl; _posLenVal = new_frame; _ctl_dbl_val = new_ctrl_val; }
-    
+  PendingOperationItem(CtrlVal* ctrl_v, bool selected /*= false*/, PendingOperationType type = SelectAudioCtrlVal)
+    { _type = type; _audCtrlValStruct = ctrl_v; _selected = selected; }
+  PendingOperationItem(CtrlList::PasteEraseOptions opts, PendingOperationType type = SetAudioCtrlPasteEraseMode)
+    { _type = type; _audio_ctrl_paste_erase_opts = opts; }
+  PendingOperationItem(CtrlListList* cll, PendingOperationType type = UpdateAudioCtrlListGroups)
+    { _type = type; _aud_ctrl_list_list = cll; }
+  PendingOperationItem(CtrlList* cl, PendingOperationType type = UpdateAudioCtrlGroups)
+    { _type = type; _aud_ctrl_list = cl; }
+  // Type is SetAudioCtrlMoveMode (or other future simple boolean operations).
+  PendingOperationItem(int valA, int valB, int valC, PendingOperationType type)
+    { _type = type; _intA = valA; _intB = valB; _intC = valC;}
+
   
   // Takes ownership of the original list so it can be deleted in the non-RT stage.
   PendingOperationItem(TempoList* orig_tempo_l, TempoList* new_tempo_l, PendingOperationType type = ModifyTempoList)
@@ -531,11 +579,14 @@ struct PendingOperationItem
   PendingOperationItem(MetroAccentsMap** old_map, MetroAccentsMap* new_map, PendingOperationType type = ModifyMetronomeAccentMap)
     { _type = type; _metroAccentsMap = old_map; _newMetroAccentsMap = new_map; }
 
-  // Type is SwitchMetronomeSettings, SetExternalSyncFlag, SetUseJackTransport.
+  // Type is SwitchMetronomeSettings, SetExternalSyncFlag, SetUseJackTransport, SwitchMidiRemoteSettings.
   PendingOperationItem(bool* bool_pointer, bool v, PendingOperationType type)
     { _type = type; _bool_pointer = bool_pointer; _boolA = v; }
 
-  PendingOperationItem(PendingOperationType type) // type is EnableAllAudioControllers.
+  PendingOperationItem(MidiRemote* old_remote, MidiRemote* new_remote, PendingOperationType type = ModifyMidiRemote)
+    { _type = type; _midiRemote = old_remote; _newMidiRemote = new_remote; }
+
+  PendingOperationItem(PendingOperationType type) // type is EnableAllAudioControllers, UpdateAllAudioCtrlGroups.
     { _type = type; }
 
   // Takes ownership of the original list so it can be deleted in the non-RT stage.
