@@ -41,9 +41,10 @@
 #include "track.h"
 #include "xml.h"
 
-namespace MusECore {
+// For debugging loading, saving, copy, paste, clone: Uncomment the fprintf section.
+#define DEBUG_PART_LOADSAVE(dev, format, args...) // fprintf(dev, format, ##args);
 
-int Part::snGen=0;
+namespace MusECore {
 
 //---------------------------------------------------------
 //   MidiCtrlViewState::write
@@ -170,8 +171,12 @@ void Part::unchainClone()
   // Isolate the part.
   _prevClone = this;
   _nextClone = this;
-  
-  _clonemaster_sn = this->_sn;
+
+  DEBUG_PART_LOADSAVE(stderr, "Part::unchainClone:%p %s muuid cur:%s new:%s\n",
+    this, name().toUtf8().constData(),
+    _clonemaster_uuid.toString().toUtf8().constData(), _uuid.toString().toUtf8().constData());
+
+  _clonemaster_uuid = this->_uuid;
 }
 
 void Part::chainClone(Part* p)
@@ -195,13 +200,24 @@ void Part::chainClone(Part* p)
   
   // we only chain clones. we must trust in the GUI thread that the eventlist is consistent.
   
-  this->_clonemaster_sn = p->_sn;
+  DEBUG_PART_LOADSAVE(stderr, "Part::chainClone:%p %s part:%p %s muuid cur:%s new:%s\n",
+    this, name().toUtf8().constData(),
+    p, p->name().toUtf8().constData(),
+    _clonemaster_uuid.toString().toUtf8().constData(), p->_uuid.toString().toUtf8().constData());
+
+  this->_clonemaster_uuid = p->clonemaster_uuid();
 }
 
 void Part::rechainClone()
 {
     if(_backupClone)
     {
+      DEBUG_PART_LOADSAVE(stderr, "Part::rechainClone:%p %s muuid:%s _backupClone:%p %s muuid:%s calling chainClone...\n",
+        this, name().toUtf8().constData(),
+        _clonemaster_uuid.toString().toUtf8().constData(),
+        _backupClone, _backupClone->name().toUtf8().constData(),
+        _backupClone->clonemaster_uuid().toString().toUtf8().constData());
+
         this->chainClone(_backupClone);
         _backupClone = nullptr;
     }
@@ -209,7 +225,7 @@ void Part::rechainClone()
 
 bool Part::isCloneOf(const Part* other) const
 {
-	return this->_clonemaster_sn == other->_clonemaster_sn;
+	return this->_clonemaster_uuid == other->_clonemaster_uuid;
 }
 
 int Part::nClones() const
@@ -400,6 +416,16 @@ Part* PartList::find(int idx)
       return nullptr;
       }
 
+Part* PartList::findCloneMaster(const QUuid& cloneUuid) const
+{
+  for(const_iterator ip = cbegin(); ip != cend(); ++ip)
+  {
+    if(ip->second->clonemaster_uuid() == cloneUuid)
+      return ip->second;
+  }
+  return nullptr;
+}
+
 //---------------------------------------------------------
 //   Part
 //---------------------------------------------------------
@@ -410,12 +436,12 @@ Part::Part(Track* t)
       _prevClone = this;
       _nextClone = this;
       _backupClone = nullptr;
-      _sn = newSn();
-      _clonemaster_sn = _sn;
       _track      = t;
       _selected   = false;
       _mute       = false;
       _colorIndex = 0;
+      _uuid = newUuid();
+      _clonemaster_uuid = _uuid;
       }
 
 Part::~Part()
@@ -429,6 +455,13 @@ Part::~Part()
       }  
 }
 
+QUuid Part::uuid() const { return _uuid; }
+void Part::setUuid(const QUuid& uuid) { _uuid = uuid; }
+QUuid Part::newUuid() const { return QUuid::createUuid(); }
+QUuid Part::clonemaster_uuid() const { return _clonemaster_uuid; }
+void Part::setClonemasterUuid(const QUuid& uuid) { _clonemaster_uuid = uuid; }
+
+
 WavePart* WavePart::duplicateEmpty() const
 {
 	WavePart* part = new WavePart((WaveTrack*)this->_track);
@@ -703,7 +736,7 @@ void Part::splitPart(unsigned int tickpos, Part*& p1, Part*& p2) const
 
 void Song::changePart(Part* oPart, Part* nPart)
       {
-      nPart->setSn(oPart->sn());
+      nPart->setUuid(oPart->uuid());
 
       Track* oTrack = oPart->track();
       Track* nTrack = nPart->track();
@@ -882,18 +915,5 @@ bool WavePart::closeAllEvents()
   }
   return closed;
 }
-      
-//---------------------------------------------------------
-//   ClonePart
-//---------------------------------------------------------
-
-ClonePart::ClonePart(const Part* p, int i) 
-{
-  cp = p;
-  id = i;
-  is_deleted = false;
-  _uuid = QUuid::createUuid();
-}
-
 
 } // namespace MusECore
