@@ -2,6 +2,7 @@
 #include <iostream>
 
 #include "scripts.h"
+#include "track.h"
 #include "part.h"
 #include "song.h"
 #include "debug.h"
@@ -57,7 +58,9 @@ void Scripts::executeScript(QWidget *parent, const char* scriptfile, PartList* p
     progress.setRange(0,parts->size());
     progress.setValue(0);
     progress.setCancelButton(nullptr);
-    MusEGlobal::song->startUndo(); // undo this entire block
+
+    Undo ops;
+
     for (const auto& i : *parts) {
         //const char* tmp = tmpnam(NULL);
         char tmp[16] = "muse-tmp-XXXXXX";
@@ -81,6 +84,14 @@ void Scripts::executeScript(QWidget *parent, const char* scriptfile, PartList* p
         writeStringToFile(fp,tempStr);
         sprintf(tempStr, "QUANTLEN %d\n", quant);
         writeStringToFile(fp,tempStr);
+        MidiTrack *track = part->track();
+        if (track->type() == Track::MIDI)
+        {
+	        sprintf(tempStr, "TYPE MIDI\n");
+        } else if (track->type() == Track::DRUM) {
+	        sprintf(tempStr, "TYPE DRUM\n");
+        }
+        writeStringToFile(fp,tempStr);
 
         if (MusEGlobal::debugMsg)
             std::cout << "Events in part " << part->events().size() << std::endl;
@@ -98,18 +109,14 @@ void Scripts::executeScript(QWidget *parent, const char* scriptfile, PartList* p
                 sprintf(tempStr,"NOTE %d %d %d %d\n", ev.tick(), ev.dataA(),  ev.lenTick(), ev.dataB());
                 writeStringToFile(fp,tempStr);
 
-                // Operation is undoable but do not start/end undo.
                 // Indicate do not do port controller values and clone parts.
-                MusEGlobal::song->applyOperation(UndoOp(UndoOp::DeleteEvent,
-                                                        ev, part, false, false), Song::OperationUndoable);
+                ops.push_back(UndoOp(UndoOp::DeleteEvent, ev, part, false, false));
 
             } else if (ev.type()==Controller) {
                 sprintf(tempStr,"CONTROLLER %d %d %d %d\n", ev.tick(), ev.dataA(), ev.dataB(), ev.dataC());
                 writeStringToFile(fp,tempStr);
-                // Operation is undoable but do not start/end undo.
                 // Indicate do not do port controller values and clone parts.
-                MusEGlobal::song->applyOperation(MusECore::UndoOp(MusECore::UndoOp::DeleteEvent,
-                                                                  ev, part, false, false), Song::OperationUndoable);
+                ops.push_back(UndoOp(UndoOp::DeleteEvent, ev, part, false, false));
             }
         }
         fclose(fp);
@@ -125,7 +132,6 @@ void Scripts::executeScript(QWidget *parent, const char* scriptfile, PartList* p
         if (myProcess->exitCode()) {
             QMessageBox::warning(parent, tr("MusE - external script failed"),
                                  tr("MusE was unable to launch the script, error message:\n%1").arg(QString(errStr)));
-            MusEGlobal::song->endUndo(SC_EVENT_REMOVED);
             return;
         }
         if (errStr.size()> 0) {
@@ -162,26 +168,24 @@ void Scripts::executeScript(QWidget *parent, const char* scriptfile, PartList* p
                     e.setPitch(pitch);
                     e.setVelo(velo);
                     e.setLenTick(len);
-                    // Operation is undoable but do not start/end undo.
                     // Indicate do not do port controller values and clone parts.
-                    MusEGlobal::song->applyOperation(UndoOp(UndoOp::AddEvent,
-                                                            e, part, false, false), Song::OperationUndoable);
+                    ops.push_back(UndoOp(UndoOp::AddEvent, e, part, false, false));
                 }
                 if (line.startsWith("CONTROLLER"))
                 {
                     QStringList sl = line.split(" ");
 
                     Event e(Controller);
+                    int tick = sl[1].toInt();
                     int a = sl[2].toInt();
                     int b = sl[3].toInt();
                     int c = sl[4].toInt();
+                    e.setTick(tick);
                     e.setA(a);
                     e.setB(b);
                     e.setC(c);
-                    // Operation is undoable but do not start/end undo.
                     // Indicate do not do port controller values and clone parts.
-                    MusEGlobal::song->applyOperation(UndoOp(UndoOp::AddEvent,
-                                                            e, part, false, false), Song::OperationUndoable);
+                    ops.push_back(UndoOp(UndoOp::AddEvent, e, part, false, false));
                 }
             }
             file.close();
@@ -192,10 +196,10 @@ void Scripts::executeScript(QWidget *parent, const char* scriptfile, PartList* p
         progress.setValue(progress.value()+1);
     } // for
 
-    MusEGlobal::song->endUndo(SC_EVENT_REMOVED);
+    // Operation is undoable but do not start/end undo.
+    MusEGlobal::song->applyOperationGroup(ops);
 }
 
-
 void Scripts::populateScriptMenu(QMenu* menuScripts)
 {
     menuScripts->clear();
