@@ -258,6 +258,8 @@ enum {
     SHRT_MUTE_CURRENT_TRACKS,
     SHRT_SOLO_CURRENT_TRACKS,
 
+    SHRT_TOGGLE_TRACK_HEIGHTS,
+
     //To be in arranger, pianoroll & drumeditor. p4.0.10 now globally handled, too.
     SHRT_SELECT_ALL, //Ctrl+A
     SHRT_SELECT_NONE, //Ctrl+Shift+A
@@ -420,7 +422,7 @@ enum {
     SHRT_PREV_MARKER,
 
     // Arranger
-    SHRT_PART_NORMALIZE, // Alt+N
+    SHRT_PART_NORMALIZE, // Alt+Shift+N
 
     SHRT_TOOL_STRETCH,
     SHRT_TOOL_SAMPLERATE,
