@@ -346,7 +346,7 @@ void SigList::tickValues(unsigned t, int* bar, int* beat, unsigned* tick) const
       {
       ciSigEvent e = upper_bound(t);
       if (e == end()) {
-            fprintf(stderr, "tickValues(0x%x) not found(%zd)\n", t, size());
+            fprintf(stderr, "tickValues(0x%x) not found(%ld)\n", t, (long unsigned int) size());
             *bar = 0;
             *beat = 0;
             *tick = 0;
