@@ -42,7 +42,8 @@
 #include "lock_free_buffer.h"
 #include "mpevent.h"
 #include "wave.h"
-
+#include "ctrl.h"
+#include "midiremote.h"
 
 //#define IPC_EVENT_FIFO_SIZE ( std::min( std::max(size_t(256), size_t(MusEGlobal::segmentSize * 16)),  size_t(16384)) )
 
@@ -137,10 +138,28 @@ class Song : public QObject {
       long _xRunsCount;
 
       // Receives events from any threads. For now, specifically for creating new
-      //  controllers in the gui thread and adding them safely to the controller lists.
+      //  midi controllers in the gui thread and adding them safely to the controller lists.
       LockFreeMPSCRingBuffer<MidiPlayEvent> *_ipcInEventBuffers;
       LockFreeMPSCRingBuffer<MidiPlayEvent> *_ipcOutEventBuffers;
       
+      // Receives messages about audio controller changes from any thread, such as assigning
+      //  midi messages to an audio controller from inside the audio thread.
+      // This supports more information than just a song changed signal, so that drawing updates
+      //  and item updates can be optimized to only that which is needed.
+      LockFreeMPSCRingBuffer<CtrlGUIMessage> *_ipcCtrlGUIMessages;
+
+      // Whether an opening BeginAudioCtrlMoveMode command has been run.
+      // A corresponding closing EndAudioCtrlMoveMode should always follow at some point.
+      bool _audioCtrlMoveModeBegun;
+
+      // A list of midi input assignments to the song.
+      // For midi control of various controllers and other items, not necessarily requiring
+      //  a specific track (for example it might affect only selected tracks).
+      // There is also a list in class Track for controllers and various things.
+      MidiAudioCtrlMap _midiAssignments;
+
+      MidiRemote _midiRemote;
+
       // Used for fastforward and fastrewind states (currently only through MIDI remote control)
       FastMove _fastMove = NORMAL_MOVEMENT;
       bool loopFlag;
@@ -187,6 +206,17 @@ class Song : public QObject {
       // Fills operations if given, otherwise creates and executes its own operations list.
       void processTrackAutomationEvents(AudioTrack *atrack, Undo* operations = 0);
 
+   protected:
+      // Internal routine for preparing the operation that ends audio controller movement mode.
+      // Called from the Undo system's stage 1 only.
+      bool audioCtrlMoveEnd(PendingOperationList& operations);
+      // Internal routine for preparing to undo the operation that ends audio controller movement mode.
+      // Called from the Undo system's stage 1 only.
+      bool undoAudioCtrlMoveEnd(PendingOperationList& operations);
+      // Process any special IPC audio thread - to - gui thread messages.
+      // Called by gui thread only. Returns true on success.
+      bool processIpcCtrlGUIMessages();
+
    public:
       Song(const char* name = 0);
       ~Song();
@@ -251,6 +281,9 @@ class Song : public QObject {
       int globalPitchShift() const      { return _globalPitchShift; }
       void setGlobalPitchShift(int val) { _globalPitchShift = val; }
       
+      // Returns the list of midi input assignments to the song.
+      MidiAudioCtrlMap* midiAssignments();
+
       // REMOVE Tim. samplerate. Added. TODO
 #if 0
       // Set the project's sample rate. This can be different than the current actual sample rate.
@@ -421,7 +454,7 @@ class Song : public QObject {
       
       Track* findTrack(const Part* part) const;
       Track* findTrack(const QString& name) const;
-      bool trackExists(Track* t) const { return _tracks.find(t) != _tracks.cend(); }
+      bool trackExists(const Track* t) const { return _tracks.find(t) != _tracks.cend(); }
 
       void setRecordFlag(Track*, bool val, Undo* operations = 0);
       // This is a non- realtime safe operation mainly used when loading files, while the engine is paused or idle.
@@ -447,7 +480,9 @@ class Song : public QObject {
       // Fills operations if given, otherwise creates and executes its own operations list.
       void processAutomationEvents(Undo* operations = 0);
       void processMasterRec();
-      int execAutomationCtlPopup(AudioTrack*, const QPoint&, int);
+      // The track can be an audio or midi track and cannot be NULL.
+      // Returns -1 if aborted or error.
+      int execAutomationCtlPopup(Track*, const QPoint&, MidiAudioCtrlStruct::IdType idType, int id);
       int execMidiAutomationCtlPopup(MidiTrack*, MidiPart*, const QPoint&, int);
       bool connectJackRoutes(const MusECore::Route& src, const MusECore::Route& dst, bool disconnect = false);
       void connectAudioPorts();
@@ -469,6 +504,23 @@ class Song : public QObject {
       // Called by audio thread only. Returns true on success.
       bool processIpcOutEventBuffers();
 
+      // Put a message into the IPC audio controller ring buffer for the gui thread to process.
+      bool putIpcCtrlGUIMessage(const CtrlGUIMessage& msg);
+
+      // Returns false if nothing was added to operations, or error.
+      bool collectAudioCtrlPasteModeOps(
+        // Map of selected items.
+        AudioAutomationItemTrackMap& trackMap,
+        // List that operations will be added to.
+        MusECore::Undo& operations,
+        // Erase options.
+        const MusECore::CtrlList::PasteEraseOptions& eraseOpts,
+        // Whether to recover items erased from previous moves.
+        bool recoverErasedItems = false,
+        // Whether the paste is a move or a copy.
+        bool isCopy = false
+        );
+
       //-----------------------------------------
       //   undo, redo, operation groups
       //-----------------------------------------
@@ -489,13 +541,24 @@ class Song : public QObject {
       void addUndo(UndoOp i);
       void setUndoRedoText();
 
+      // Returns true if audio controller move mode has begun (a BeginAudioCtrlMoveMode command was run).
+      bool audioCtrlMoveModeBegun() const;
+      // Convenience method for pushing SetAudioCtrlMoveMode (true) and BeginAudioCtrlMoveMode commands onto the undo list.
+      // Called from graphics thread only, in response to a condition such as dragging then dropping graph points.
+      void beginAudioCtrlMoveMode(Undo&) const;
+      // Common convenience method for pushing SetAudioCtrlMoveMode (false) and EndAudioCtrlMoveMode commands onto the undo list.
+      // Called from graphics thread only, in response to various conditions such as clicking on empty background
+      //  or manually ending the mode by popup menu.
+      void endAudioCtrlMoveMode(Undo&) const;
+
       //-----------------------------------------
       //   Configuration
       //-----------------------------------------
 
       SynthI* createSynthI(const QString& sclass, const QString& uri, const QString& label = QString(),
                            Synth::Type type = Synth::SYNTH_TYPE_END, Track* insertAt = 0);
-      
+      MidiRemote* midiRemote();
+
       //-----------------------------------------
       //   Debug
       //-----------------------------------------
@@ -575,11 +638,14 @@ class Song : public QObject {
       void quantizeChanged(bool);
       void markerChanged(int);
       void midiNote(int pitch, int velo);  
-      void controllerChanged(MusECore::Track*, int); 
+      void controllerChanged(
+        const MusECore::Track*, int clId, unsigned int frame = 0,
+        CtrlGUIMessage::Type type = CtrlGUIMessage::PAINT_UPDATE);
       void newPartsCreated(const std::map< const MusECore::Part*, std::set<const MusECore::Part*> >&);
       void sigDirty();
       void recModeChanged(int);
       void cycleModeChanged(int);
+      void midiLearnReceived(const MidiRecordEvent&);
       };
 
 } // namespace MusECore
