@@ -429,7 +429,7 @@ WaveEdit::WaveEdit(MusECore::PartList* pl, QWidget* parent, const char* name)
       
       connect(canvas, SIGNAL(toolChanged(int)), tools2, SLOT(set(int)));
       connect(tools2, SIGNAL(toolChanged(int)), canvas,   SLOT(setTool(int)));
-      connect(MusEGlobal::muse, &MusE::configChanged, tools2, &EditToolBar::configChanged);
+      _configChangedTools2MetaConn = connect(MusEGlobal::muse, &MusE::configChanged, tools2, &EditToolBar::configChanged);
 
       connect(hscroll, SIGNAL(scrollChanged(int)), canvas, SLOT(setXPos(int)));
       connect(hscroll, SIGNAL(scaleChanged(int)),  canvas, SLOT(setXMag(int)));
@@ -605,6 +605,7 @@ void WaveEdit::setTime(unsigned samplepos)
 WaveEdit::~WaveEdit()
       {
       DEBUG_WAVEEDIT(stderr, "WaveEdit dtor\n");
+      disconnect(_configChangedTools2MetaConn);
       }
 
 //---------------------------------------------------------
@@ -745,6 +746,11 @@ void WaveEdit::songChanged1(MusECore::SongChangedStruct_t bits)
         if(bits & SC_TRACK_REMOVED)
           checkTrackInfoTrack();
         
+        songChanged(bits);
+
+        if (parts()->empty())
+          return;
+
         if (bits & SC_DIVISION_CHANGED)
         {
           // The division has changed. The raster table and raster model will have been
@@ -762,16 +768,14 @@ void WaveEdit::songChanged1(MusECore::SongChangedStruct_t bits)
           // Now set a reasonable zoom (mag) range.
           setupHZoomRange();
         }
-        
+
         if (bits & SC_SOLO)
         {
           MusECore::WavePart* part = (MusECore::WavePart*)(parts()->begin()->second);
           solo->blockSignals(true);
           solo->setChecked(part->track()->solo());
           solo->blockSignals(false);
-        }  
-        
-        songChanged(bits);
+        }
 
         // We'll receive SC_SELECTION if a different part is selected.
         // Addition - also need to respond here to moving part to another track. (Tim)
