@@ -36,6 +36,7 @@
 #include "audioprefetch.h"
 //#include "latency_compensator.h"
 #include "config.h"
+#include "xml_statistics.h"
 
 // Turn on some cool terminal 'peak' meters for debugging
 //  presence of actual audio at various places
@@ -252,13 +253,17 @@ void WaveTrack::fetchData(unsigned pos, unsigned samples, float** bp, bool doSee
 //   write
 //---------------------------------------------------------
 
-void WaveTrack::write(int level, Xml& xml) const
+void WaveTrack::write(int level, Xml& xml, XmlWriteStatistics* stats) const
       {
+      XmlWriteStatistics locStats;
+      if(!stats)
+        stats = &locStats;
+
       xml.tag(level++, "wavetrack");
       AudioTrack::writeProperties(level, xml);
       const PartList* pl = cparts();
       for (ciPart p = pl->begin(); p != pl->end(); ++p)
-            p->second->write(level, xml);
+            p->second->write(level, xml, false, false, stats);
       xml.etag(level, "wavetrack");
       }
 
@@ -266,8 +271,12 @@ void WaveTrack::write(int level, Xml& xml) const
 //   read
 //---------------------------------------------------------
 
-void WaveTrack::read(Xml& xml)
+void WaveTrack::read(Xml& xml, XmlReadStatistics* stats)
       {
+      XmlReadStatistics locStats;
+      if(!stats)
+        stats = &locStats;
+
       for (;;) {
             Xml::Token token = xml.parse();
             const QString& tag = xml.s1();
@@ -278,7 +287,7 @@ void WaveTrack::read(Xml& xml)
                   case Xml::TagStart:
                         if (tag == "part") {
                               Part* p = 0;
-                              p = Part::readFromXml(xml, this);
+                              p = Part::readFromXml(xml, this, stats, false, true);
                               if(p)
                                 parts()->add(p);
                               }
@@ -289,6 +298,7 @@ void WaveTrack::read(Xml& xml)
                         break;
                   case Xml::TagEnd:
                         if (tag == "wavetrack") {
+                              fixOldColorScheme();
                               mapRackPluginsToControllers();
                               goto out_of_WaveTrackRead_forloop;
                               }
@@ -429,7 +439,7 @@ bool WaveTrack::getPrefetchData(
         if(corr_frame_end_pos <= pos)
         {
           if(MusEGlobal::debugMsg)
-            fprintf(stderr, "fifo get(%s) (A) error expected %ld, got %ld\n", name().toLocal8Bit().constData(), frame_pos, pos);
+            fprintf(stderr, "fifo get(%s) (A) error expected %ld, got %ld\n", name().toLocal8Bit().constData(), (long int) frame_pos, (long int) pos);
           return false;
         }
       }
@@ -504,7 +514,7 @@ bool WaveTrack::getPrefetchData(
       if(pos != expect_nextpos)
       {
         if(MusEGlobal::debugMsg)
-          fprintf(stderr, "fifo get(%s) (B) error expected %ld, got %ld\n", name().toLocal8Bit().constData(), expect_nextpos, pos);
+          fprintf(stderr, "fifo get(%s) (B) error expected %ld, got %ld\n", name().toLocal8Bit().constData(), (long int) expect_nextpos, (long int) pos);
         return false;
       }
 
@@ -641,6 +651,9 @@ bool WaveTrack::getInputData(unsigned pos, int channels, unsigned nframes,
 
 bool WaveTrack::getData(unsigned framePos, int dstChannels, unsigned nframe, float** bp)
 {
+  if(off())
+    return false;
+
   bool have_data = false;
   
   const bool track_rec_flag = recordFlag();
@@ -669,13 +682,21 @@ bool WaveTrack::getData(unsigned framePos, int dstChannels, unsigned nframe, flo
     
     // The data retrieved by this will already be latency compensated.
     have_data = getInputData(framePos, dstChannels, nframe, used_in_chan_array, bp);
-    
+
+// Reference counting diagnostics.
+//     if(!_recFile.isNull())
+//       fprintf(stderr, "WaveTrack::getData before recfile() ref count:%d\n", _recFile.getRefCount());
+
     // Do we want to record the incoming data?
     if(have_data && track_rec_flag && 
       (MusEGlobal::audio->isRecording() ||
        (MusEGlobal::song->record() && MusEGlobal::extSyncFlag && MusEGlobal::midiSyncContainer.isPlaying())) && 
       recFile())
     {
+
+// Reference counting diagnostics.
+//       fprintf(stderr, "WaveTrack::getData after recfile() ref count:%d\n", _recFile.getRefCount());
+
       if(MusEGlobal::audio->freewheel())
       {
       }
