@@ -44,7 +44,9 @@
 #ifdef VST_NATIVE_SUPPORT
 
 #ifdef VST_SDK_QUIRK_DEF
+#ifndef __cdecl
 #define __cdecl
+#endif // __cdecl
 #endif
 
 //#include "aeffectx.h"
