@@ -26,31 +26,31 @@
 #include "simpler_plugin.h"
 
 #include "lilv/lilv.h"
-#include "lv2/lv2plug.in/ns/ext/data-access/data-access.h"
-#include "lv2/lv2plug.in/ns/ext/state/state.h"
-#include "lv2/lv2plug.in/ns/ext/atom/atom.h"
-#include "lv2/lv2plug.in/ns/ext/midi/midi.h"
-#include "lv2/lv2plug.in/ns/ext/buf-size/buf-size.h"
+#include "lv2/data-access/data-access.h"
+#include "lv2/state/state.h"
+#include "lv2/atom/atom.h"
+#include "lv2/midi/midi.h"
+#include "lv2/buf-size/buf-size.h"
 #ifdef LV2_EVENT_BUFFER_SUPPORT
-#include "lv2/lv2plug.in/ns/ext/event/event.h"
+#include "lv2/event/event.h"
 #endif
-#include "lv2/lv2plug.in/ns/ext/options/options.h"
-#include "lv2/lv2plug.in/ns/ext/parameters/parameters.h"
-#include "lv2/lv2plug.in/ns/ext/patch/patch.h"
-#include "lv2/lv2plug.in/ns/ext/port-groups/port-groups.h"
-#include "lv2/lv2plug.in/ns/ext/presets/presets.h"
-#include "lv2/lv2plug.in/ns/ext/state/state.h"
-#include "lv2/lv2plug.in/ns/ext/time/time.h"
+#include "lv2/options/options.h"
+#include "lv2/parameters/parameters.h"
+#include "lv2/patch/patch.h"
+#include "lv2/port-groups/port-groups.h"
+#include "lv2/presets/presets.h"
+#include "lv2/state/state.h"
+#include "lv2/time/time.h"
 #ifdef LV2_URI_MAP_SUPPORT
-#include "lv2/lv2plug.in/ns/ext/uri-map/uri-map.h"
+#include "lv2/uri-map/uri-map.h"
 #endif
-#include "lv2/lv2plug.in/ns/ext/urid/urid.h"
-#include "lv2/lv2plug.in/ns/ext/worker/worker.h"
-#include "lv2/lv2plug.in/ns/ext/port-props/port-props.h"
-#include "lv2/lv2plug.in/ns/ext/atom/forge.h"
-#include "lv2/lv2plug.in/ns/ext/log/log.h"
-#include "lv2/lv2plug.in/ns/extensions/ui/ui.h"
-#include "lv2/lv2plug.in/ns/ext/dynmanifest/dynmanifest.h"
+#include "lv2/urid/urid.h"
+#include "lv2/worker/worker.h"
+#include "lv2/port-props/port-props.h"
+#include "lv2/atom/forge.h"
+#include "lv2/log/log.h"
+#include "lv2/ui/ui.h"
+#include "lv2/dynmanifest/dynmanifest.h"
 #include "lv2extui.h"
 #include "lv2extprg.h"
 
@@ -182,11 +182,11 @@ struct LV2MidiPort
 
 enum LV2ControlPortType
 {
-   LV2_PORT_DISCRETE = 1,
-   LV2_PORT_INTEGER,
+   LV2_PORT_INTEGER = 1,
    LV2_PORT_CONTINUOUS,
    LV2_PORT_LOGARITHMIC,
-   LV2_PORT_TRIGGER
+   LV2_PORT_TRIGGER,
+   LV2_PORT_ENUMERATION
 };
 
 struct LV2ControlPort
